/*************************************************************************
 *
 *  $RCSfile: binarydecide.cxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: lla $ $Date: 2001/06/01 08:01:37 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <osl/time.h>
#include <rtl/ustring.hxx>

#include "binarydecide.hxx"
#include "filehelper.hxx"
#include "localsession.hxx"
#include "tracer.hxx"

namespace configmgr
{
	using namespace osl;

#define ASCII(x) rtl::OUString::createFromAscii(x)

// -----------------------------------------------------------------------------
// This Memberfunctions helps to decide which file is to load, the xml or the binary file.
// -----------------------------------------------------------------------------
	bool OFileTimeStampDecision::isBTimeGreaterATime(TimeValue const& A, TimeValue const& B) const
	{
		if (B.Seconds > A.Seconds) return true;
		if (B.Seconds < A.Seconds) return false;
		
		// Seconds are equal, check Nanoseconds
		if (B.Nanosec > A.Nanosec) return true;
		
		// lower or equal
		return false;
	}
// -----------------------------------------------------------------------------
	
	ODecideContext OFileTimeStampDecision::getNewerFile(ODecideContext const& _aContextA, ODecideContext const& _aContextB) const
	{
		// PRE: _sFileA and _sFileB must exist.
		
		TimeValue aFileStampA = {0,0};
		TimeValue aFileStampB = {0,0};
		
		aFileStampA = FileHelper::getFileModificationStamp(_aContextA.sFileURL);
		aFileStampB = FileHelper::getFileModificationStamp(_aContextB.sFileURL);
		
		if (isBTimeGreaterATime(aFileStampA, aFileStampB))
		{
			return _aContextB;
		}
		return _aContextA;
	}
// -----------------------------------------------------------------------------
	ODecideContext OFileTimeStampDecision::checkFileStamp(PathHelper const& _aPath)
	{
		ODecideContext aContext[2];
		
		sal_Int16 nCount = 0;
		
		rtl::OUString sXMLFileURL = _aPath.createFileURL();
		if (FileHelper::fileExist(sXMLFileURL))
		{
			aContext[nCount++] = ODecideContext(sXMLFileURL, ODecideContext::XML_FILE);
		}
		
		rtl::OUString sBinaryFileURL = _aPath.createBinaryFileURL();
		if (FileHelper::fileExist(sBinaryFileURL))
		{
			aContext[nCount++] = ODecideContext(sBinaryFileURL, ODecideContext::BINARY_FILE);
		}
		
		if (nCount == 0) return ODecideContext();     // NO File exist
		if (nCount == 1) return aContext[0];		  // only one File exist
		
		ODecideContext aContextBack;
		aContextBack = getNewerFile(aContext[0], aContext[1]);	// youngest of 2 Files
		// if (nCount > 2)
		// {
		// 	   aContextBack = getNewerFile(aContextBack, aContext[2]);	// youngest of 3 Files
		// }
		return aContextBack;
	}
	// -----------------------------------------------------------------------------
	rtl::OUString OFileTimeStampDecision::getFileURL()
	{
		m_aContext = checkFileStamp(m_aPath);
		return m_aContext.sFileURL;
	}
	// -----------------------------------------------------------------------------
	ODecideContext::Enum OFileTimeStampDecision::getFileType()
	{
		if (m_aContext.eType == ODecideContext::NO_FILE)
		{
			CFG_TRACE_WARNING("No file found, Path: %s Module %s", OUSTRING2ASCII(m_aPath.getPath()), OUSTRING2ASCII(m_aPath.getModule()));
		}
		return m_aContext.eType;
	}
} // namespace
