/*************************************************************************
 *
 *  $RCSfile: pathhelper.cxx,v $
 *
 *  $Revision: 1.7 $
 *
 *  last change: $Author: jb $ $Date: 2001/07/05 17:05:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include "pathhelper.hxx"

#ifndef _CONFIGMGR_FILEHELPER_HXX_
#include "filehelper.hxx"
#endif
#ifndef CONFIGMGR_NAMEHELPER_HXX
#include "namehelper.hxx"
#endif
#ifndef _CONFIGMGR_TRACER_HXX_
#include "tracer.hxx"
#endif

#ifndef _RTL_USTRING_HXX_
#include <rtl/ustring.hxx>
#endif
#ifndef _OSL_FILE_HXX_
#include <osl/file.hxx>
#endif

#ifndef _COM_SUN_STAR_IO_IOEXCEPTION_HPP_
#include <com/sun/star/io/IOException.hpp>
#endif

#ifndef INCLUDED_SET
#include <set>
#define INCLUDED_SET
#endif

namespace configmgr
{
	namespace css = com::sun::star;
	namespace io = css::io;
    using rtl::OUString;
#define ASCII(x) rtl::OUString::createFromAscii(x)

// -----------------------------------------------------------------------------

// -----------------------------------------------------------------------------
	static inline rtl::OUString makeString(sal_Unicode const ch)
	{
		return rtl::OUString(&ch, 1);
	}

// -----------------------------------------------------------------------------
	class OEnsureDirectory
	{
		rtl::OUString           m_sMinimumBasePath;
		rtl::OUString           m_sBasePath;                // should contain all til /config
		std::set<rtl::OUString> m_aPathes;

		bool ensureDirectory(rtl::OUString const& _sDirex) const throw (io::IOException);
		bool createDirectoryByGoDown(rtl::OUString const& _aDirectory) const throw (io::IOException);
		
		rtl::OUString dropLastDirectory(rtl::OUString const& _sDirex) const;

	public:
		explicit OEnsureDirectory(rtl::OUString const& _aBasePath)
				:m_sBasePath(_aBasePath)
			{
				// PRE: the path to the _aFullPath MUST already exist
				m_sMinimumBasePath = dropLastDirectory(m_sBasePath);
#ifdef DEBUG
				// Check if BasePath exists
#endif
			}
		
		void directoryExist(rtl::OUString const& _aFilename) throw (io::IOException);
		~OEnsureDirectory();
	};

    // -----------------------------------------------------------------------------
	// rtl::OUString OEnsureDirectory::m_sFileDelimiter = rtl::OUString(FileHelper::getFileDelimiter(), 1);
	// rtl::OUString OEnsureDirectory::m_sFileDelimiter = makeString( FileHelper::getFileDelimiter() );

	bool OEnsureDirectory::ensureDirectory(rtl::OUString const& _sDirexURL) const throw (io::IOException)
	{
		// direct create a directory 
		osl::FileBase::RC eError = osl::Directory::create(_sDirexURL); // try to create the directory
		if (eError == osl::FileBase::E_EXIST) return true; // Exist
		if (eError == osl::FileBase::E_None) return true;  // hopeful created
		return false;
	}
	// -----------------------------------------------------------------------------
	bool OEnsureDirectory::createDirectoryByGoDown(rtl::OUString const& _aDirectory) const throw (io::IOException)
	{
		// TODO check, if Directory is a good Directory

		// Never try to go under the allready must exist directory path
		// off the configmgr registry structure because endless loops are possible

		if (m_sMinimumBasePath.equals(_aDirectory))
		{
			throw io::IOException(ASCII("createDirectory: the minimal directory depth is arrived, can't create directory"), NULL);
		}

		bool bRet = ensureDirectory(_aDirectory);
		if (bRet == false)
		{
			rtl::OUString aParentDirectory = dropLastDirectory(_aDirectory);
			// short check, if there is something in the String (ParentDirectory) to prevent us for
			// endless loops because dropLastDirectory will never fail and ensureDirectory also.
			if (aParentDirectory.getLength() > 0)
			{
				bRet = createDirectoryByGoDown(aParentDirectory);
				bRet = ensureDirectory(_aDirectory);
			}
		}
		return bRet;
	}
	// -----------------------------------------------------------------------------
	void OEnsureDirectory::directoryExist(rtl::OUString const& _aDirectory) throw (io::IOException)
	{
		// ensure that the given directory exist, if not, create, if problemes, throw away
		
		if (m_aPathes.find(_aDirectory) == m_aPathes.end()) // sDirex not found in set<Direx>
		{
			OSL_ENSURE(m_sBasePath.getLength() > 0, "ERROR: BasePath not set?");

			try 
			{
				if (createDirectoryByGoDown(_aDirectory) == true)
				{
					m_aPathes.insert(_aDirectory);
				}
				else
				{
					// Problem to create the directory.
					// don't try to handle here.
				}
			}
			catch (io::IOException)
			{
				// hmmm...?
				throw;
			}
		}
	}

// -----------------------------------------------------------------------------
	rtl::OUString OEnsureDirectory::dropLastDirectory(rtl::OUString const& _sDirex) const
	{
		// goto last '/' and cut the rest.
		sal_Int32 nIdx = _sDirex.lastIndexOf(ASCII("/"), _sDirex.getLength());
		if (nIdx > 0)
		{
			return _sDirex.copy(0, nIdx);
		}
		return _sDirex;
	}
// -----------------------------------------------------------------------------
	OEnsureDirectory::~OEnsureDirectory()
	{
		m_aPathes.clear();
	}

// -----------------------------------------------------------------------------
// --------------------------------- PathHelper ---------------------------------
// -----------------------------------------------------------------------------
	PathHelper::PathHelper()
			:m_bUseNewFileStructure(false),
			 m_pDirectoryGuarantee(NULL)
	{
	}
// -----------------------------------------------------------------------------
	PathHelper::~PathHelper()
	{
		delete m_pDirectoryGuarantee;
	}
	
// -----------------------------------------------------------------------------
	void PathHelper::ensure_DirectoryGuarantee()
	{
		//! MemoryLeak
		OSL_ASSERT(m_pDirectoryGuarantee);
	}
// -----------------------------------------------------------------------------
	void PathHelper::setPath(rtl::OUString const& _sPath)
	{
		m_sPath = _sPath;
		OSL_ENSURE(m_sPath.getLength() > 0, "Path Length isn't ok.");
		OSL_ENSURE(m_pDirectoryGuarantee == 0, "There is a pointer already there. Problems?");
		delete m_pDirectoryGuarantee;
		m_pDirectoryGuarantee = new OEnsureDirectory(m_sPath);
	}
// -----------------------------------------------------------------------------
	void PathHelper::setModule(rtl::OUString const& _sModule)
	{
		if (m_sPath.getLength() == 0) return;        // SetupSession
		m_sModule = _sModule;

		ensure_DirectoryGuarantee();

		OSL_ENSURE(m_sPath.getLength() > 0, "Path is deleted by ensure_Directory...");
		
		rtl::OUString aFileURL(createFileURL());
		m_pDirectoryGuarantee->directoryExist(PathHelper::convertFilenameToDirectory(aFileURL));

		// rtl::OUString aBinFileURL(createBinaryFileURL());
		// m_pDirectoryGuarantee->directoryExist(PathHelper::convertFilenameToDirectory(aBinFileURL));
	}
// -----------------------------------------------------------------------------
	void PathHelper::checkDirStructure()
	{
		// convert filename to real URL
		static OUString sFileDelimiter = makeString( FileHelper::getFileDelimiter() );

		OSL_ENSURE(m_sPath.getLength() > 0, "SharePath isn't set.");
		OUString sDirex(m_sPath);
		sDirex += sFileDelimiter;
		sDirex += m_sInstance;

		if (FileHelper::directoryExist(sDirex) == true)
			m_bUseNewFileStructure = true;
		else
			m_bUseNewFileStructure = false;
	}
// -----------------------------------------------------------------------------
	OUString PathHelper::getFileURL(const OUString& _aModuleName, const OUString &_aFullPath, rtl::OUString const& _aExtraDirectory, rtl::OUString const& _sExtension) const
	{
		OUString aFullname; //  = _aFullPath;
		sal_Unicode const chFileDelimiter = FileHelper::getFileDelimiter();
		static OUString sFileDelimiter;
		if (sFileDelimiter.getLength() == 0)
		{
			sFileDelimiter = OUString(&chFileDelimiter, 1);
		}
		
		if (_aExtraDirectory.getLength() > 0)
		{
			aFullname += sFileDelimiter;
			aFullname += _aExtraDirectory;
		}
		
		if (m_bUseNewFileStructure)
		{
			if (m_sInstance.getLength() > 0)
			{
				aFullname += sFileDelimiter;
				aFullname += m_sInstance;
			}
		}
		
		//! HACK: add FileDelimiter only if _aModuleName don't starts with '/'
		if (_aModuleName[0] != sal_Unicode('/'))
		{
			aFullname += sFileDelimiter;
		}

		if (m_bUseNewFileStructure)
		{
			aFullname += NameHelper::convertDotsToSlash(_aModuleName);
		}
		else
		{
			aFullname += _aModuleName;
		}
		aFullname += _sExtension;
		
//!		OEnsureDirectory aGuarantee(_aFullPath);
//!		aGuarantee.directoryExist(aFullname);

		// convert filename to real URL
		return _aFullPath + aFullname; // FileHelper::convertFilenameToFileURL(_aFullPath + aFullname);
	}
	// -----------------------------------------------------------------------------

	OUString PathHelper::createFileURL() const
	{
		static OUString sXMLExtension(ASCII(".xml"));
		static OUString sExtraDirectory;
		return getFileURL(m_sModule, m_sPath, sExtraDirectory, sXMLExtension);
	}
	OUString PathHelper::createBinaryFileURL() const
	{
		static OUString sExtension(ASCII(".dat"));
		static OUString sExtraDirectory(ASCII("cache"));
		return getFileURL(m_sModule, m_sPath, sExtraDirectory, sExtension);
	}
	OUString PathHelper::createFileUpdateURL() const
	{
		static OUString sExtension(ASCII(".xml"));
		static OUString sExtraDirectory;
		static OUString sUpdate(ASCII("_update"));
		OUString aUpdateName = m_sModule + sUpdate;
		return getFileURL(aUpdateName, m_sPath, sExtraDirectory, sExtension);
	}

// -----------------------------------------------------------------------------
	OUString PathHelper::createDirectory() const
	{
		return m_sPath + createDirectory(ASCII(""));
	}
// -----------------------------------------------------------------------------
	OUString PathHelper::createBinaryDirectory() const
	{
		return m_sPath + createDirectory(ASCII("cache"));
	}
	
// -----------------------------------------------------------------------------
	OUString PathHelper::createDirectory(rtl::OUString const& _sExtraDirectory) const
	{
		OUString aFullname; //  = _aFullPath;
		sal_Unicode const chFileDelimiter = FileHelper::getFileDelimiter();
		static OUString sFileDelimiter;
		if (sFileDelimiter.getLength() == 0)
		{
			sFileDelimiter = OUString(&chFileDelimiter, 1);
		}
		
		if (_sExtraDirectory.getLength() > 0)
		{
			aFullname += sFileDelimiter;
			aFullname += _sExtraDirectory;
		}
		
		if (m_bUseNewFileStructure)
		{
			if (m_sInstance.getLength() > 0)
			{
				aFullname += sFileDelimiter;
				aFullname += m_sInstance;
			}
		}
		return aFullname;
	}

// -----------------------------------------------------------------------------
	// static
	rtl::OUString PathHelper::convertFilenameToDirectory(rtl::OUString const& _sFilename)
	{
		// goto last '/' and cut the rest.
		sal_Int32 nIdx = _sFilename.lastIndexOf(ASCII("/"), _sFilename.getLength());
		if (nIdx > 0)
		{
			return _sFilename.copy(0, nIdx);
		}
		return _sFilename;
	}
	// -----------------------------------------------------------------------------
	void PathHelper::checkDirectory(rtl::OUString const& _sDirectoryURL)
	{
		ensure_DirectoryGuarantee();

		OSL_ENSURE(m_sPath.getLength() > 0, "Path is deleted by ensure_Directory...");
		
		m_pDirectoryGuarantee->directoryExist(_sDirectoryURL);
	}
}
