/*************************************************************************
 *
 *  $RCSfile: javaldx.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: jl $ $Date: 2001/06/22 09:38:34 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <osl/file.h>
#include <osl/profile.hxx>
#include <osl/security.hxx>
#include <sal/types.h>
#include <rtl/ustring.hxx>
#include <tools/solar.h>


using namespace rtl;
using namespace osl;
/* avoid linking against libcrypt, since crypt is 
 * not really needed here */
char *
crypt(const char *key, const char *salt)
{
	return (char*)key;
}

/* separate the leading path component from pFile */
static sal_Char*
getDirnameFromFile (sal_Char* pFile)
{
	const sal_Char *pDot = ".";
	sal_Char *pLastSlash;

	/* Find last '/'.  */
	pLastSlash = strrchr (pFile, '/');

	if ( pLastSlash == pFile )
		/* The last slash is the first char, return "/" */
		++pLastSlash;

	if (pLastSlash != NULL)
		/* Terminate the path.  */
		pLastSlash[0] = '\0';
	else
		strncpy (pFile, pDot, strlen( pDot ) + 1 );

	return pFile;
}

/* check whether pPath exists and is a directory */
static sal_Bool
isDirectory( sal_Char* pPath )
{
	struct stat aStatus;

	if (   (stat( pPath, &aStatus ) == 0)
		&& (S_ISDIR(aStatus.st_mode)) )
		return sal_True;

	return sal_False;
}

/* checks whether pPathOrFile is an existing file or directory. If its
 * a file, it is modified to uncover its path component */
static sal_Bool
getDirectory ( sal_Char* pPathOrFile )
{
	if ( isDirectory( pPathOrFile ) )
		return sal_True;
	else
	if ( isDirectory( getDirnameFromFile ( pPathOrFile ) ) )
		return sal_True;
	else
		return sal_False;
}

int
main ( int argc, char **argv )
{
	Security aSecurity;
	OUString usSversionrcPath;
	// get home directory
	if( ! aSecurity.getConfigDir( usSversionrcPath))
		return -1;
	OUString usSversionFile( RTL_CONSTASCII_USTRINGPARAM( "/.sversionrc"));
	usSversionrcPath += usSversionFile;
	// open a profile with /<home>/.sversionrc and get the installation directory
	Profile profile( usSversionrcPath);
	OString sOfficeDir= profile.readString( OString( RTL_CONSTASCII_STRINGPARAM("Versions")),
											OString( RTL_CONSTASCII_STRINGPARAM(STAROFFICE_SVERSION_KEY )),
						OString( ));
	// build the path for the javarc file
	OString sJavarcPath= sOfficeDir + OString( RTL_CONSTASCII_STRINGPARAM("/user/config/javarc"));

	// check if the javarc file exists with this path
	if ( access( sJavarcPath.pData->buffer, R_OK ) == -1 )
	{
		// try if /<install dir>/share/config/javarc exists
		sJavarcPath= sOfficeDir + OString( RTL_CONSTASCII_STRINGPARAM("/share/config/javarc"));		
		if ( access(sJavarcPath.pData->buffer, R_OK ) == -1 )
			return -1;
	}

	// Open a profile with the javarc path and read the entry "JavaLibPath"
	OUString usCompleteProfileName= OStringToOUString( sJavarcPath, RTL_TEXTENCODING_ASCII_US);
	Profile profJavarc( usCompleteProfileName);
	OString sLibPath= profJavarc.readString( OString( RTL_CONSTASCII_STRINGPARAM("Java")),
						OString( RTL_CONSTASCII_STRINGPARAM("JavaLibPath")),
						OString( ));

	// finally print the paths which are used for Java to stdout
	if( sLibPath.getLength() > 0)
		fprintf (stdout, "%s\n", sLibPath.getStr() );
	else
		return -1;
	return 0;
}


