/*************************************************************************
 *
 *  $RCSfile: fltdlg.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: pb $ $Date: 2001/06/28 11:44:17 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

//_________________________________________________________________________________________________________________
//	my own includes
//_________________________________________________________________________________________________________________

#include "classes/fltdlg.hxx"
#include "fltdlg.hrc"

//_________________________________________________________________________________________________________________
//	interface includes
//_________________________________________________________________________________________________________________

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

//_________________________________________________________________________________________________________________
//	includes of other projects
//_________________________________________________________________________________________________________________

#ifndef _LIST_HXX //autogen
#include <tools/list.hxx>
#endif
#ifndef _URLOBJ_HXX //autogen
#include <tools/urlobj.hxx>
#endif
#ifndef _MSGBOX_HXX //autogen
#include <vcl/msgbox.hxx>
#endif
#ifndef _BUTTON_HXX //autogen
#include <vcl/button.hxx>
#endif
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif

#pragma hdrstop

using namespace ::rtl;
using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::beans;
using namespace ::com::sun::star::container;

namespace framework
{

String FilterDialog::GetUserSelectedFilter(
	Window* pParent,
	::com::sun::star::uno::Sequence< ::com::sun::star::beans::PropertyValue >& rMediaDescriptor,
	::com::sun::star::uno::Reference< ::com::sun::star::container::XNameAccess >& rXNameAccess,
	const String* pDefFilter )
{
	String aTypeName;

	::vos::OGuard aSolarGuard( Application::GetSolarMutex() );
	{
		// execute not thread safe VCL code
		ResMgr* pResManager = CREATEVERSIONRESMGR( fwk );

		if ( pResManager )
		{
			String aDefFilterString;
			FilterDialog aFilterDialog( pParent,
										pResManager,
										rMediaDescriptor,
										rXNameAccess,
										pDefFilter );
			short nResult = aFilterDialog.Execute();
			if ( nResult == RET_OK )
			{
				aTypeName = aFilterDialog.GetSelectEntry();
			}
		}

		delete pResManager;
	}

	return aTypeName;
}

// -----------------------------------------------------------------------

FilterDialog::FilterDialog
(
	Window* pParent,
	ResMgr*	pResMgr,
	Sequence< PropertyValue >& rMediaDescriptor,
	Reference< XNameAccess >& rXNameAccess,
	const String* pDefFilter
) :

	ModalDialog( pParent, ResId( DLG_FILTER_SELECT, pResMgr )),
	m_rXNameAccess( rXNameAccess ),
	m_rMediaDescriptor( rMediaDescriptor ),

	aMimeTypeFL	( this, ResId( FL_MIMETYPE ) ),
	aMIMEType	( this, ResId( FT_DLG_MIMETYPE ) ),
	aExtension	( this, ResId( FT_DLG_EXTENSTION ) ),
	aFilterFL	( this, ResId( FL_FILTER ) ),
	aListbox	( this, ResId( LB_DLG_LISTBOX ) ),
	aOkBtn		( this, ResId( LB_DLG_OKBTN ) ),
	aCancelBtn	( this, ResId( LB_DLG_CANCELBTN ) ),
	aHelpBtn	( this, ResId( BTN_HELP ) )

{
	const short nExtra = *(short*)GetClassRes();
	IncrementRes(sizeof(short));
	FreeResource();

	String aMimeText( aMIMEType.GetText() );
	String aExtensionText( aExtension.GetText() );

	for ( int i=0; i < m_rMediaDescriptor.getLength(); i++ )
	{
		OUString aName = m_rMediaDescriptor[i].Name;
		if ( aName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "ContentType" )) )
		{
			OUString aString;

			if ( m_rMediaDescriptor[i].Value >>= aString )
			{
				aMimeText += String( aString );
				aMIMEType.SetText( aMimeText );
			}
		}
		else if ( aName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "FileName" )) )
		{
			OUString aString;

			if ( m_rMediaDescriptor[i].Value >>= aString )
			{
				INetURLObject aURLObject( aString );
				aExtensionText += aURLObject.GetExtension();
				aExtension.SetText( aExtensionText );
			}
		}
	}

	m_FilterNames = m_rXNameAccess->getElementNames();

	aListbox.SetDoubleClickHdl(LINK(this, FilterDialog, OkHdl));

	const OUString* pFilterArray = m_FilterNames.getConstArray();
	OUString	aFilterName;
	OUString	aUIName;
	for( int n = 0; n < m_FilterNames.getLength(); n++ )
	{
		aFilterName = pFilterArray[n];

		Sequence< PropertyValue > aFilterProperties;
		Any a = rXNameAccess->getByName( aFilterName );

		a >>= aFilterProperties;
		for ( int m = 0; m < aFilterProperties.getLength(); m++ )
		{
			if ( aFilterProperties[m].Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "UIName" )) )
			{
				aFilterProperties[m].Value >>= aUIName;
				aListbox.InsertEntry( aUIName );
				break;
			}
		}
	}

	aListbox.SelectEntryPos( 0);
}

//-------------------------------------------------------------------------

String FilterDialog::GetSelectEntry()
{
	String aEntry = aListbox.GetSelectEntry();
	if ( aEntry.Len() > 0 )
	{
		try
		{
			int nPos = aListbox.GetSelectEntryPos();

			OUString				  aType;
			Sequence< PropertyValue > aFilterProperties;
			Any						  a = m_rXNameAccess->getByName( m_FilterNames[nPos] );

			a >>= aFilterProperties;
			// set type name on media descriptor
			for ( int m = 0; m < aFilterProperties.getLength(); m++ )
			{
				if ( aFilterProperties[m].Name.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "Type" )) )
				{
					sal_Bool bTypeNameFound = sal_False;
					for ( int i=0; i < m_rMediaDescriptor.getLength(); i++ )
					{
						OUString aName = m_rMediaDescriptor[i].Name;
						if ( aName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "TypeName" )) )
						{
							bTypeNameFound = sal_True;
							m_rMediaDescriptor[i].Value = aFilterProperties[m].Value;
							aFilterProperties[m].Value >>= aType;
							break;
						}
					}

					// type name is not inside media descriptor => we have to insert it!
					if ( !bTypeNameFound )
					{
						int nMediaDescriptorCount = m_rMediaDescriptor.getLength();
						m_rMediaDescriptor.realloc( nMediaDescriptorCount+1 );
						PropertyValue* pMediaDescriptorPropArray = m_rMediaDescriptor.getArray();

						PropertyValue aTypeNameProperty;

						a <<= aFilterProperties[m].Value;
						aTypeNameProperty.Name = OUString( RTL_CONSTASCII_USTRINGPARAM( "TypeName" ));
						aTypeNameProperty.Value = a;
						pMediaDescriptorPropArray[nMediaDescriptorCount] = aTypeNameProperty;
					}
					break;
				}
			}

			// set filter name on media descriptor
			sal_Bool bFilterNameFound = sal_False;
			for ( int i=0; i < m_rMediaDescriptor.getLength(); i++ )
			{
				OUString aName = m_rMediaDescriptor[i].Name;
				if ( aName.equalsAsciiL( RTL_CONSTASCII_STRINGPARAM( "FilterName" )) )
				{
					bFilterNameFound = sal_True;
					a <<= m_FilterNames[nPos];
					m_rMediaDescriptor[i].Value = a;
					break;
				}
			}

			// filter name is not inside media descriptor => we have to insert it!
			if ( !bFilterNameFound )
			{
				int nMediaDescriptorCount = m_rMediaDescriptor.getLength();
				m_rMediaDescriptor.realloc( nMediaDescriptorCount+1 );
				PropertyValue* pMediaDescriptorPropArray = m_rMediaDescriptor.getArray();

				PropertyValue aFilterNameProperty;

				a <<= m_FilterNames[nPos];
				aFilterNameProperty.Name = OUString( RTL_CONSTASCII_USTRINGPARAM( "FilterName" ));
				aFilterNameProperty.Value = a;
				pMediaDescriptorPropArray[nMediaDescriptorCount] = aFilterNameProperty;
			}

			return aType;
		}
		catch ( RuntimeException& )
		{
		}
		catch ( NoSuchElementException& )
		{
		}
		catch ( com::sun::star::lang::WrappedTargetException& )
		{
		}
	}

	return String();
}
//-------------------------------------------------------------------------

IMPL_LINK_INLINE_START( FilterDialog, OkHdl, Control *, pControl )
{
	EndDialog(RET_OK);
	return 0;
}
IMPL_LINK_INLINE_END( FilterDialog, OkHdl, Control *, pControl )


}
