/*************************************************************************
 *
 *  $RCSfile: Adler32.cxx,v $
 *
 *  $Revision: 1.3 $
 *
 *  last change: $Author: mtg $ $Date: 2001/04/19 14:13:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): Martin Gallwey (gallwey@sun.com)
 *
 *
 ************************************************************************/
#ifndef _ADLER32_HXX
#include <Adler32.hxx>
#endif
#ifndef _ZLIB_H
#include <external/zlib/zlib.h>
#endif

/**
 * A class that can be used to compute the Adler-32 checksum of a data
 * stream. An Adler-32 checksum is almost as reliable as a CRC-32 but
 * can be computed much faster.
 *
 */

using namespace rtl;
using namespace com::sun::star;

Adler32::Adler32() : nAdler(1) 
{ 
}

Adler32::~Adler32()
{
}
/** Updates checksum with specified byte
 */
void SAL_CALL Adler32::updateByte (sal_Int8 nByte)
	throw(uno::RuntimeException)
{
	sal_uInt8 pBuf[1];
	pBuf[0] = (sal_uInt8)nByte;
	nAdler  = adler32(nAdler, pBuf, 1);
}
/** Updates checksum with specified sequence of bytes
 */
void SAL_CALL Adler32::updateSegment(const uno::Sequence< sal_Int8 > &b, 
									 sal_Int32 off, 
									 sal_Int32 len)
	throw(uno::RuntimeException)
{
	nAdler = adler32(nAdler, (const unsigned char*)b.getConstArray()+off, len );
}
/** Updates checksum with specified sequence of bytes
 */
void SAL_CALL Adler32::update(const uno::Sequence< sal_Int8 > &b)
	throw(uno::RuntimeException)
{
	nAdler = adler32(nAdler, (const unsigned char*)b.getConstArray(),b.getLength());
}
/** Returns checksum value.
 */
sal_Int32 SAL_CALL Adler32::getValue()
	throw(uno::RuntimeException)
{
	return (sal_Int32) nAdler & 0xFFFFFFFFL;
}
/** Resets checksum to initial value.
 */
void SAL_CALL Adler32::reset()
	throw(uno::RuntimeException)
{
	nAdler=1;
}
