/*************************************************************************
 *
 *  $RCSfile: XclExpStream.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: dr $ $Date: 2001/02/28 06:55:12 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//___________________________________________________________________

#ifndef _SC_XCLEXPSTREAM_HXX
#include "XclExpStream.hxx"
#endif

#ifndef _EXCDEFS_HXX
#include "excdefs.hxx"
#endif

//___________________________________________________________________

XclExpStream::XclExpStream( SvStream& rOutStrm, ULONG _nMaxRecLen ) :
	rStrm( rOutStrm ),
	nMaxRecLen( _nMaxRecLen ),
	nMaxContLen( _nMaxRecLen ),
	nMaxLen( 0 ),
	nMaxSliceLen( 0 ),
	nCalcLen( 0 ),
	nHeaderLen( 0 ),
	nCurrLen( 0 ),
	nSliceLen( 0 ),
	nLastLenPos( 0 ),
	bInRec( FALSE )
{
}

XclExpStream::~XclExpStream()
{
}

void XclExpStream::InitRecord( UINT16 nRecNum )
{
	rStrm.Seek( STREAM_SEEK_TO_END );
	rStrm << nRecNum;

	nLastLenPos = rStrm.Tell();
	nHeaderLen = Min( nCalcLen, nMaxLen );
	rStrm << (UINT16) nHeaderLen;
	nCurrLen = nSliceLen = 0;
}

void XclExpStream::UpdateRecLen()
{
	if( nCurrLen != nHeaderLen )
	{
		rStrm.Seek( nLastLenPos );
		rStrm << (UINT16) nCurrLen;
	}
}

void XclExpStream::UpdateLenVars( ULONG nLen )
{
	nCurrLen += nLen;

	if( nMaxSliceLen )
	{
		nSliceLen += nLen;
		DBG_ASSERT( nSliceLen <= nMaxSliceLen, "XclExpStream::UpdateLenVars - slice overwritten" );
		if( nSliceLen >= nMaxSliceLen )
			nSliceLen = 0;
	}
}

void XclExpStream::StartContinue()
{
	UpdateRecLen();
	nMaxLen = nMaxContLen;
	nCalcLen -= nCurrLen;
	InitRecord( EXC_CONT );
}

void XclExpStream::PrepareWrite( ULONG nLen )
{
	if( !bInRec ) return;

	if( (nCurrLen + nLen > nMaxLen) ||
		(nMaxSliceLen && !nSliceLen && (nCurrLen + nMaxSliceLen > nMaxLen)) )
		StartContinue();
	UpdateLenVars( nLen );
}

ULONG XclExpStream::PrepareWrite()
{
	if( !bInRec )
		return 0;

	if( (nCurrLen >= nMaxLen) ||
		(nMaxSliceLen && !nSliceLen && (nCurrLen + nMaxSliceLen > nMaxLen)) )
		StartContinue();
	UpdateLenVars( 0 );

	return nMaxSliceLen ? nMaxSliceLen - nSliceLen : nMaxLen - nCurrLen;
}

void XclExpStream::StartRecord( UINT16 nRecNum, ULONG nRecLen )
{
	DBG_ASSERT( !bInRec, "XclExpStream::StartRecord - another record still open" );
	nMaxContLen = nMaxLen = nMaxRecLen;
	nCalcLen = nRecLen;
	bInRec = TRUE;
	InitRecord( nRecNum );
	SetSliceLen( 0 );
}

void XclExpStream::EndRecord()
{
	DBG_ASSERT( bInRec, "XclExpStream::EndRecord - no record open" );
	UpdateRecLen();
	rStrm.Seek( STREAM_SEEK_TO_END );
	bInRec = FALSE;
}

void XclExpStream::SetSliceLen( ULONG nLen )
{
	nMaxSliceLen = nLen;
	nSliceLen = 0;
}

ULONG XclExpStream::Write( const void* pData, ULONG nBytes )
{
	if( !pData || !nBytes )
		return 0;

	ULONG nRet = 0;
	if( bInRec )
	{
		const UINT8* pBuffer = (const UINT8*) pData;
		ULONG nBytesLeft = nBytes;
		BOOL bValid = TRUE;

		while( bValid && nBytesLeft )
		{
			ULONG nWriteLen = Min( PrepareWrite(), nBytesLeft );
			ULONG nWriteRet = rStrm.Write( pBuffer, nWriteLen );
			bValid = (nWriteLen == nWriteRet);
			DBG_ASSERT( bValid, "XclExpStream::Write - stream write error" );
			pBuffer += nWriteRet;
			nRet += nWriteRet;
			nBytesLeft -= nWriteRet;
			UpdateLenVars( nWriteRet );
		}
	}
	else
		nRet = rStrm.Write( pData, nBytes );

	return nRet;
}

void XclExpStream::WriteRawZeroBytes( ULONG nBytes )
{
	const ULONG nData = 0;
	ULONG nBytesLeft = nBytes;
	while( nBytesLeft >= sizeof(ULONG) )
	{
		rStrm << nData;
		nBytesLeft -= sizeof(ULONG);
	}
	if( nBytesLeft )
		rStrm.Write( &nData, nBytesLeft );
}

void XclExpStream::WriteZeroBytes( ULONG nBytes )
{
	if( !nBytes ) return;

	if( bInRec )
	{
		ULONG nBytesLeft = nBytes;
		while( nBytesLeft )
		{
			ULONG nWriteLen = Min( PrepareWrite(), nBytesLeft );
			WriteRawZeroBytes( nWriteLen );
			nBytesLeft -= nWriteLen;
			UpdateLenVars( nWriteLen );
		}
	}
	else
		WriteRawZeroBytes( nBytes );
}

ULONG XclExpStream::CopyFromStream( SvStream& rInStrm, ULONG nBytes )
{
	ULONG nStreamPos = rInStrm.Tell();
	ULONG nStreamLen = rInStrm.Seek( STREAM_SEEK_TO_END );
	rInStrm.Seek( nStreamPos );

	ULONG nBytesLeft = Min( nBytes, nStreamLen - nStreamPos );
	if( !nBytesLeft )
		return 0;

	const ULONG nMaxBuffer = 0x00001000;
	UINT8* pBuffer = new UINT8[ Min( nBytesLeft, nMaxBuffer ) ];
	ULONG nRet = 0;
	BOOL bValid = TRUE;

	while( bValid && nBytesLeft )
	{
		ULONG nWriteLen = Min( nBytesLeft, nMaxBuffer );
		rInStrm.Read( pBuffer, nWriteLen );
		ULONG nWriteRet = Write( pBuffer, nWriteLen );
		bValid = (nWriteLen == nWriteRet);
		nRet += nWriteRet;
		nBytesLeft -= nWriteRet;
	}

	delete[] pBuffer;
	return nRet;
}

void XclExpStream::WriteUnicodeBuffer( const UINT16* pBuffer, ULONG nChars, UINT8 nFlags )
{
	SetSliceLen( 0 );
	if( !pBuffer || !nChars ) return;

	ULONG nCharLen = (nFlags & EXC_STR_16BIT) ? 2 : 1;
	for( ULONG nIndex = 0; nIndex < nChars; nIndex++ )
	{
		if( bInRec && (nCurrLen + nCharLen > nMaxLen) )		// repeat flags
		{
			StartContinue();
			operator<<( nFlags );
		}
		if( nCharLen == 2 )
			operator<<( pBuffer[ nIndex ] );
		else
			operator<<( (UINT8) pBuffer[ nIndex ] );
	}
}

void XclExpStream::WriteByteStringBuffer( const ByteString& rString, UINT16 nMaxLen )
{
	SetSliceLen( 0 );
	Write( rString.GetBuffer(), Min( rString.Len(), (xub_StrLen) nMaxLen ) );
}

// ER: #71367# Xcl has an obscure sense of whether starting a new record or not,
// and crashes if it encounters the string header at the very end of a record.
// Thus we add 1 to give some room, seems like they do it that way but with another count (10?)
void XclExpStream::WriteByteString( const ByteString& rString, UINT16 nMaxLen, BOOL b16BitCount )
{
	SetSliceLen( 0 );
	UINT16 nLen = (UINT16) Min( rString.Len(), (xub_StrLen) nMaxLen );
	if( !b16BitCount )
		nLen = Min( nLen, (UINT16) 0x00FF );

	ULONG nLeft = PrepareWrite();
	if( bInRec && (nLeft <= (b16BitCount ? 2UL : 1UL)) )
		StartContinue();

	if( b16BitCount )
		operator<<( nLen );
	else
		operator<<( (UINT8) nLen );
	Write( rString.GetBuffer(), nLen );
}

ULONG XclExpStream::SetStreamPos( ULONG nPos )
{
	DBG_ASSERT( !bInRec, "XclExpStream::SetStreamPos - not allowed inside of a record" );
	return bInRec ? 0 : rStrm.Seek( nPos );
}

