/*************************************************************************
 *
 *  $RCSfile: XclExpStream.hxx,v $
 *
 *  $Revision: 1.1 $
 *
 *  last change: $Author: dr $ $Date: 2001/02/26 06:50:01 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SC_XCLEXPSTREAM_HXX
#define _SC_XCLEXPSTREAM_HXX

#ifndef _TOOLS_DEBUG_HXX
#include <tools/debug.hxx>
#endif
#ifndef _STRING_HXX
#include <tools/string.hxx>
#endif
#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

//___________________________________________________________________
// output stream class for Excel export
// - CONTINUE record handling
// - ByteString and UniString support

//___________________________________________________________________

class XclExpStream
{
private:
	SvStream&					rStrm;			// output stream

								// length data
	ULONG						nMaxRecLen;		// maximum length of a record
	ULONG						nMaxContLen;	// maximum length of CONTINUE recs
	ULONG						nMaxLen;		// current maximum
	ULONG						nMaxSliceLen;	// maximum length of data slices
	ULONG						nCalcLen;		// calculated length
	ULONG						nHeaderLen;		// length written in last record header
	ULONG						nCurrLen;		// bytes written in current record
	ULONG						nSliceLen;		// bytes written in current slice

								// stream position data
	ULONG						nLastLenPos;	// position of length in header

	BOOL						bInRec;			// TRUE if inside of a record

								// write header data, internal setup
	void						InitRecord( UINT16 nRecNum );
								// write correct length
	void						UpdateRecLen();
								// recalculate <nCurrLen> and <nSliceLeft>
	void						UpdateLenVars( ULONG nLen );
								// write CONTINUE header, internal setup
	void						StartContinue();
								// refresh counter vars, create CONTINUE recs
	void						PrepareWrite( ULONG nLen );
								// create CONTINUE rec at end, return maximum data block size
	ULONG						PrepareWrite();

								// write a coherent block of zero bytes
	void						WriteRawZeroBytes( ULONG nBytes );

public:
								XclExpStream( SvStream& rOutStrm, ULONG _nMaxRecLen );

								~XclExpStream();

								// write header data, store calculated record length
	void						StartRecord( UINT16 nRecNum, ULONG nRecLen );
								// check and correct real record length
	void						EndRecord();

								// return maximum length of a record
	inline ULONG				GetMaxRecLen() const		{ return nMaxRecLen; }
								// set maximum length (only for this record)
	inline void					SetMaxRecLen( ULONG nMax )	{ nMaxLen = nMax; }
								// set maximum length of CONTINUEs (only for this record)
	inline void					SetMaxContLen( ULONG nMax )	{ nMaxContLen = nMax; }
								// set data slice length, 0 -> no slices
	void						SetSliceLen( ULONG nLen );

	inline XclExpStream&		operator<<( char rValue );
	inline XclExpStream&		operator<<( BYTE rValue );
	inline XclExpStream&		operator<<( short rValue );
	inline XclExpStream&		operator<<( USHORT rValue );
	inline XclExpStream&		operator<<( int rValue );
	inline XclExpStream&		operator<<( unsigned int rValue );
	inline XclExpStream&		operator<<( long rValue );
	inline XclExpStream&		operator<<( ULONG rValue );
	inline XclExpStream&		operator<<( float rValue );
	inline XclExpStream&		operator<<( double rValue );

								// write <nBytes> bytes
	ULONG						Write( const void* pData, ULONG nBytes );
								// write <nBytes> zero bytes
	void						WriteZeroBytes( ULONG nBytes );
								// copy <nBytes> bytes from current position of <rInStrm>
	ULONG						CopyFromStream( SvStream& rInStrm, ULONG nBytes = STREAM_SEEK_TO_END );

	// *** unicode string export is realized with helper class XclExpUniString ***
	// (slice length setting has no effect here -> disabled automatically)

								// write buffer as 8/16 bit, repeat <nFlags> at start of a CONTINUE
	void						WriteUnicodeBuffer( const UINT16* pBuffer, ULONG nChars, UINT8 nFlags );

	// *** write 8-bit-strings *** (slice length setting has no effect here -> disabled automatically)

								// write ByteString buffer
	void						WriteByteStringBuffer(
									const ByteString& rString,
									UINT16 nMaxLen = 0x00FF );
								// writes 8/16 bit length and ByteString buffer
	void						WriteByteString(
									const ByteString& rString,
									UINT16 nMaxLen = 0x00FF,
									BOOL b16BitCount = FALSE );

	// SvStream functions
	ULONG						SetStreamPos( ULONG nPos );
	inline ULONG				GetStreamPos() const { return rStrm.Tell(); }
};

inline XclExpStream& XclExpStream::operator<<( char nValue )
{
	PrepareWrite( sizeof(char) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( BYTE nValue )
{
	PrepareWrite( sizeof(char) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( short nValue )
{
	PrepareWrite( sizeof(short) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( USHORT nValue )
{
	PrepareWrite( sizeof(USHORT) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( int nValue )
{
	PrepareWrite( sizeof(int) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( unsigned int nValue )
{
	PrepareWrite( sizeof(unsigned int) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( long nValue )
{
	PrepareWrite( sizeof(long) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( ULONG nValue )
{
	PrepareWrite( sizeof(ULONG) );
	rStrm << nValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( float fValue )
{
	PrepareWrite( sizeof(float) );
	rStrm << fValue;
	return *this;
}

inline XclExpStream& XclExpStream::operator<<( double fValue )
{
	PrepareWrite( sizeof(double) );
	rStrm << fValue;
	return *this;
}


#endif

