/*************************************************************************
 *
 *  $RCSfile: SchTransferable.cxx,v $
 *
 *  $Revision: 1.10 $
 *
 *  last change: $Author: bm $ $Date: 2001/07/13 16:03:49 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include "SchTransferable.hxx"

#include "schmod.hxx"

#ifndef _SVDMODEL_HXX
#include <svx/svdmodel.hxx>
#endif
#ifndef _SVDVIEW_HXX
#include <svx/svdview.hxx>
#endif
#ifndef _SVDPAGE_HXX
#include <svx/svdpage.hxx>
#endif

#ifndef _SVDOGRAF_HXX 
#include <svx/svdograf.hxx>
#endif
#ifndef _CPPUHELPER_QUERYINTERFACE_HXX_
#include <cppuhelper/queryinterface.hxx>
#endif
// header for class OGuard
#ifndef _VOS_MUTEX_HXX_
#include <vos/mutex.hxx>
#endif
// header for class Application
#ifndef _SV_SVAPP_HXX
#include <vcl/svapp.hxx>
#endif
#ifndef _UNTOOLS_UCBSTREAMHELPER_HXX
#include <unotools/ucbstreamhelper.hxx>
#endif

#ifndef _COM_SUN_STAR_BEANS_XPROPERTYSET_HPP_
#include <com/sun/star/beans/XPropertySet.hpp>
#endif

#define SCH_TRANSFER_OBJECTTYPE_OLE			0x00000001
#define SCH_TRANSFER_OBJECTTYPE_DRAWMODEL   0x00000002

using namespace ::com::sun::star;

// CTOR for clipboard
SchTransferable::SchTransferable( SdrModel* pClipboardModel,
                                  const TransferableObjectDescriptor& rObjDesc ) :
        mpContentModel   ( pClipboardModel ),
        mpDragSourceView ( NULL ),
		maObjDesc        ( rObjDesc ),
        mpGraphic        ( NULL )
{
	CreateData();
}

// CTOR for drag and drop
SchTransferable::SchTransferable( SdrModel* pDragSrcModel,
                                  SdrView*  pDragSrcView,
                                  const TransferableObjectDescriptor& rObjDesc ) :
        mpContentModel   ( pDragSrcModel ),
        mpDragSourceView ( pDragSrcView ),
		maObjDesc        ( rObjDesc ),
        mpGraphic        ( NULL )
{
	CreateData();
}

SchTransferable::~SchTransferable()
{
	ObjectReleased();

	::vos::OGuard aGuard( Application::GetSolarMutex());

    if( mpContentModel )
        delete mpContentModel;

    if( mpDragSourceView )
        delete mpDragSourceView;

    //ToDo: is DoClose() necessary?
	if( maDocShellRef.Is() )
	{
//  		SvEmbeddedObject* pObj = maDocShellRef;
//  		SchChartDocShell* pDocSh = SAL_STATIC_CAST( SchChartDocShell*, pObj );
//  		pDocSh->DoClose();
	}

    // release object shell
	maDocShellRef.Clear();

	if( mpGraphic )
		delete mpGraphic;
}

//  void SchTransferable::CreateInternView()
//  {
//  	::vos::OGuard aGuard( Application::GetSolarMutex());

//      mpVDev = new VirtualDevice( *( (OutputDevice*)Application::GetDefaultDevice()) );

//      MapMode aMap = mpVDev->GetMapMode();

//      DBG_ASSERT( mpChartDocIntern, "Invalid Work Document" );
//      aMap.SetMapUnit( mpChartDocIntern->GetScaleUnit() );
//      aMap.SetScaleX( mpChartDocIntern->GetScaleFraction() );
//      aMap.SetScaleY( mpChartDocIntern->GetScaleFraction() );
//      mpVDev->SetMapMode( aMap );

//      mbHaveOwnView = TRUE;
//      mpViewIntern = new SchView( mpChartDocIntern, mpVDev );
//      mpViewIntern->EndListening( *mpChartDocIntern );
//      mpViewIntern->SetActualWin( mpVDev );
//      mpViewIntern->SetMarkHdlHidden( TRUE );

//      const Point aPos;
//      SdrPageView* pPageView = mpViewIntern->ShowPagePgNum( 0, aPos );
//      mpViewIntern->MarkAll();
//  }

void SchTransferable::CreateData()
{
    // copy to clipboard
	if( mpContentModel && ! mpDragSourceView )
	{
        ::vos::OGuard aGuard( Application::GetSolarMutex());
		const SdrPage* pPage = mpContentModel->GetPage( 0 );

		if( pPage && pPage->GetObjCount() == 1 )
		{
			SdrObject* pObj = pPage->GetObj( 0 );
			if( pObj->ISA( SdrGrafObj ))
			{
				if( mpGraphic )
					delete mpGraphic;

				mpGraphic = new Graphic( ((SdrGrafObj*) pObj )->GetTransformedGraphic());
			}
		}

        // get size of selected objects

        SdrExchangeView aView( mpContentModel );
        Point aPos;
        SdrPageView* pPv = aView.ShowPagePgNum( 0, aPos );
        aView.MarkAll( pPv );
        Size aSize = aView.GetAllMarkedRect().GetSize();
        maObjDesc.maSize = aSize;
	}
    // drag and drop
    else if( mpDragSourceView && ! mpContentModel )
	{
        ::vos::OGuard aGuard( Application::GetSolarMutex());
        const SdrMarkList&	rMarkList = mpDragSourceView->GetMarkList();

		if( rMarkList.GetMarkCount() == 1 )
		{
			const SdrObject* pObj = rMarkList.GetMark( 0 )->GetObj();

			if ( pObj && pObj->ISA( SdrGrafObj ) )
				mpGraphic = new Graphic( ((SdrGrafObj*) pObj )->GetTransformedGraphic());
		}

        mpContentModel = mpDragSourceView->GetMarkedObjModel();
	}
}

// protected - implementation of TransferableHelper methods
void SchTransferable::AddSupportedFormats()
{
	if( mpGraphic )
	{
		AddFormat( SOT_FORMATSTR_ID_SVXB );

		if( mpGraphic->GetType() == GRAPHIC_BITMAP )
		{
			AddFormat( SOT_FORMAT_BITMAP );
			AddFormat( SOT_FORMAT_GDIMETAFILE );
		}
		else
		{
			AddFormat( SOT_FORMAT_GDIMETAFILE );
			AddFormat( SOT_FORMAT_BITMAP );
		}
	}
    else
    {
//          AddFormat( SOT_FORMATSTR_ID_EMBED_SOURCE );
//          AddFormat( SOT_FORMATSTR_ID_OBJECTDESCRIPTOR );
        AddFormat( SOT_FORMATSTR_ID_DRAWING );
        AddFormat( SOT_FORMAT_GDIMETAFILE );
        AddFormat( SOT_FORMAT_BITMAP );
    }
}

sal_Bool SchTransferable::GetData( const datatransfer::DataFlavor& rFlavor )
{
	sal_uInt32	nFormat = SotExchange::GetFormat( rFlavor );
	sal_Bool	bOK = sal_False;

	if( HasFormat( nFormat ) )
	{
		if( ( nFormat == SOT_FORMATSTR_ID_LINKSRCDESCRIPTOR ||
			  nFormat == SOT_FORMATSTR_ID_OBJECTDESCRIPTOR ))
		{
            bOK = SetTransferableObjectDescriptor( maObjDesc, rFlavor );
		}
        else if ( nFormat == SOT_FORMATSTR_ID_DRAWING )
        {
            bOK = SetObject( mpContentModel, SCH_TRANSFER_OBJECTTYPE_DRAWMODEL, rFlavor );
        }
		else if( nFormat == FORMAT_GDIMETAFILE ||
                 nFormat == FORMAT_BITMAP )
        {
            SdrExchangeView aView( mpContentModel );
            Point aPos;
            SdrPageView* pPv = aView.ShowPagePgNum( 0, aPos );
            aView.MarkAll( pPv );
            if( nFormat == FORMAT_GDIMETAFILE )
            {
                bOK = SetGDIMetaFile( aView.GetAllMarkedMetaFile( TRUE ), rFlavor );
            }
            else if( nFormat == FORMAT_BITMAP )
            {
                bOK = SetBitmap( aView.GetAllMarkedBitmap( TRUE ), rFlavor );
            }
        }
		else if( nFormat == FORMAT_STRING )
		{
			bOK = SetString( String(), rFlavor );
		}
		else if( ( nFormat == SOT_FORMATSTR_ID_SVXB ) && mpGraphic )
		{
            bOK = SetGraphic( *mpGraphic, rFlavor );
		}
		else if( nFormat == SOT_FORMATSTR_ID_EMBED_SOURCE )
		{
            DBG_ERROR( "No longer supported" );
            bOK = sal_False;
		}
	}

	return bOK;	
}

sal_Bool SchTransferable::WriteObject(
	SotStorageStreamRef& rxOStm, void* pUserObject, sal_uInt32 nUserObjectId,
	const datatransfer::DataFlavor& rFlavor )
{
	sal_Bool bRet = sal_False;

	switch( nUserObjectId )
	{
		case SCH_TRANSFER_OBJECTTYPE_OLE  :
			{
                DBG_ERROR( "No longer supported" );
                bRet = sal_False;
			}
			break;

        case SCH_TRANSFER_OBJECTTYPE_DRAWMODEL:
            {
                SdrModel* pDrawModel = (SdrModel*)pUserObject;

                pDrawModel->SetStreamingSdrModel( TRUE );

                //  SdrModel stream operator doesn't support XML
                //! call XML export here!
                rxOStm->SetVersion( SOFFICE_FILEFORMAT_50 );

                rxOStm->SetBufferSize( 0xff00 );
                pDrawModel->PreSave();
                pDrawModel->GetItemPool().SetFileFormatVersion( (USHORT)rxOStm->GetVersion() );
                pDrawModel->GetItemPool().Store( *rxOStm );
                *rxOStm << *pDrawModel;
                pDrawModel->PostSave();
                rxOStm->Commit();
                pDrawModel->SetStreamingSdrModel( FALSE );
                bRet = ( rxOStm->GetError() == ERRCODE_NONE );
            }
            break;

		default:
			break;
	}

	return bRet;
}

void SchTransferable::ObjectReleased()
{
	::vos::OGuard aGuard( Application::GetSolarMutex());

	if( this == SCH_MOD1()->GetDragTransferable())
		SCH_MOD1()->SetDragTransferable( NULL );

	if( this == SCH_MOD1()->GetClipboardTransferable())
		SCH_MOD1()->SetClipboardTransferable( NULL );

    TransferableHelper::ObjectReleased();
}

// XUnoTunnel
sal_Int64 SAL_CALL SchTransferable::getSomething( const uno::Sequence< sal_Int8 >& aIdentifier )
	throw( uno::RuntimeException )
{
	if( aIdentifier.getLength() == 16 && 0 == rtl_compareMemory( getUnoTunnelId().getConstArray(),
																 aIdentifier.getConstArray(), 16 ) )
	{
		return (sal_Int64)this;
	}
	return 0;
}

// helper for XUnoTunnel
SchTransferable* SchTransferable::getImplementation( uno::Reference< uno::XInterface > xData ) throw()
{
	uno::Reference< lang::XUnoTunnel > xUT( xData, uno::UNO_QUERY );
	if( xUT.is() )
		return (SchTransferable*)xUT->getSomething( SchTransferable::getUnoTunnelId() );
	else
		return NULL;
}

// generate a uniqueId
const uno::Sequence< sal_Int8 > & SchTransferable::getUnoTunnelId() throw()
{
	static uno::Sequence< sal_Int8 > * pSeq = 0;
	if( !pSeq )
	{
		osl::Guard< osl::Mutex > aGuard( osl::Mutex::getGlobalMutex() );
		if( !pSeq )
		{
			static uno::Sequence< sal_Int8 > aSeq( 16 );
			rtl_createUuid( (sal_uInt8*)aSeq.getArray(), 0, sal_True );
			pSeq = &aSeq;
		}
	}
	return *pSeq;
}
