/*************************************************************************
 *
 *  $RCSfile: sdbinfilter.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: cl $ $Date: 2001/06/29 12:12:22 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#include <tools/urlobj.hxx>
#include <sot/formats.hxx>
#include <svtools/itempool.hxx>
#include <svtools/style.hxx>
#include <svtools/sfxecode.hxx>
#include <svtools/saveopt.hxx>
#include <sfx2/docfile.hxx>
#include <sfx2/docfilt.hxx>

#ifndef MAC
#ifndef SVX_LIGHT
#include "../../ui/inc/docshell.hxx"
#endif //!SVX_LIGHT
#else  //MAC
#ifndef SVX_LIGHT
#include "docshell.hxx"
#endif //!SVX_LIGHT
#endif //!MAC

#ifndef _SFXITEMSET_HXX 
#include <svtools/itemset.hxx>
#endif

#ifndef _XDEF_HXX 
#include <svx/xdef.hxx>
#endif

#ifndef _SVX_XFLGRIT_HXX 
#include <svx/xflgrit.hxx>
#endif
#ifndef _SVX_XBTMPIT_HXX 
#include <svx/xbtmpit.hxx>
#endif
#ifndef _SVX_XLNDSIT_HXX 
#include <svx/xlndsit.hxx>
#endif
#ifndef _SVX_XLNSTIT_HXX 
#include <svx/xlnstit.hxx>
#endif
#ifndef _SVX_XFLFTRIT_HXX 
#include <svx/xflftrit.hxx>
#endif
#ifndef _SVX_XFLHTIT_HXX 
#include <svx/xflhtit.hxx>
#endif
#ifndef _SVX_XLNEDIT_HXX 
#include <svx/xlnedit.hxx>
#endif

#include "sdpage.hxx"
#include "drawdoc.hxx"
#include "strmname.h"
#include "sdbinfilter.hxx"

// -----------
// - Defines -
// -----------

#define POOL_BUFFER_SIZE		32768
#define BASIC_BUFFER_SIZE		8192
#define DOCUMENT_BUFFER_SIZE	32768

// --------------
// - Namespaces -
// --------------

using namespace ::com::sun::star::uno;
using namespace ::com::sun::star::task;
using namespace ::com::sun::star::frame;

// ---------------
// - SdBINFilter -
// ---------------

SdBINFilter::SdBINFilter( SfxMedium& rMedium, SdDrawDocShell& rDocShell, sal_Bool bShowProgress ) :
	SdFilter( rMedium, rDocShell, bShowProgress )
{
}

// -----------------------------------------------------------------------------

SdBINFilter::~SdBINFilter()
{
}

// -----------------------------------------------------------------------------

void PostLoadCheckStyleSheetPool( SfxStyleSheetPool* pStyleSheetPool, SdDrawDocument* pDoc )
{	
	if( pStyleSheetPool )
	{
		SfxStyleSheetBase* pStyleSheet = pStyleSheetPool->First();
		while( pStyleSheet )
		{
			SfxItemSet& rSet = pStyleSheet->GetItemSet();

			const SfxPoolItem *pItem;

			// check fill gradient item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_FILLGRADIENT, sal_False, &pItem ) )
			{
				XFillGradientItem* pNewItem = ((XFillGradientItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_FILLGRADIENT );
					delete pNewItem;
				}
			}

			// check fill bitmap item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_FILLBITMAP, sal_False, &pItem ) )
			{
				XFillBitmapItem* pNewItem = ((XFillBitmapItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_FILLBITMAP );
					delete pNewItem;
				}
			}

			// check line dash item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_LINEDASH, sal_False, &pItem ) )
			{
				XLineDashItem* pNewItem = ((XLineDashItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_LINEDASH );
					delete pNewItem;
				}
			}

			// check line start item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_LINESTART, sal_False, &pItem ) )
			{
				XLineStartItem* pNewItem = ((XLineStartItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_LINESTART );
					delete pNewItem;
				}
			}

			// check line end item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_LINEEND, sal_False, &pItem ) )
			{
				XLineEndItem* pNewItem = ((XLineEndItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_LINEEND );
					delete pNewItem;
				}
			}

			// check fill float transparence item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_FILLFLOATTRANSPARENCE, sal_False, &pItem ) )
			{
				XFillFloatTransparenceItem* pNewItem = ((XFillFloatTransparenceItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_FILLFLOATTRANSPARENCE );
					delete pNewItem;
				}
			}

			// check fill hatch item for valid name
			if( SFX_ITEM_SET == rSet.GetItemState( XATTR_FILLHATCH, sal_False, &pItem ) )
			{
				XFillHatchItem* pNewItem = ((XFillHatchItem*)pItem)->checkForUniqueItem( pDoc );
				if( pNewItem != pItem )
				{
					rSet.Put( *pNewItem, XATTR_FILLHATCH );
					delete pNewItem;
				}
			}
/* sample code for fixing bullet problems. Didn't need it yet
			if( SFX_ITEM_SET == rSet.GetItemState( EE_PARA_NUMBULLET, sal_False, &pItem ) )
			{
				SvxNumRule* pRule = ((SvxNumBulletItem*)rSet.GetItem( EE_PARA_NUMBULLET ))->GetNumRule();

				const USHORT nLevelCount = pRule->GetLevelCount();
				USHORT nLevel;
				BOOL bNeedsFix = FALSE;
				for( nLevel = 0; !bNeedsFix && (nLevel < nLevelCount); nLevel++ )
				{
					const SvxNumberFormat& rLevel = pRule->GetLevel(nLevel);
					bNeedsFix = (rLevel.GetNumberingType() == com::sun::star::style::NumberingType::CHAR_SPECIAL) &&
								(rLevel.GetBulletChar() == 0 );
				}

				if( bNeedsFix )
				{
					SvxNumRule aNewRule( pRule->GetFeatureFlags(), nLevelCount, FALSE, pRule->GetNumRuleType() );

					for( nLevel = 0; !bNeedsFix && (nLevel < nLevelCount); nLevel++ )
					{
						SvxNumberFormat aLevel(pRule->GetLevel(nLevel)) ;
						if( (aLevel.GetNumberingType() == com::sun::star::style::NumberingType::CHAR_SPECIAL) && (aLevel.GetBulletChar() == 0 ) )
							aLevel.SetNumberingType( com::sun::star::style::NumberingType::NUMBER_NONE );

						aNewRule.SetLevel(nLevel, aLevel);
					}

					rSet.Put( SvxNumBulletItem( aNewRule, EE_PARA_NUMBULLET ) );

				}
			}
*/
			pStyleSheet = pStyleSheetPool->Next();
		}
	}
}

// -----------------------------------------------------------------------------

sal_Bool SdBINFilter::Import()
{
	SvStorage*	pStore = mrMedium.GetStorage();
	ULONG		nStorFmt = pStore->GetFormat();
	sal_Bool	bRet = sal_False;

	if ( nStorFmt == SOT_FORMATSTR_ID_STARIMPRESS_50 ||
		 nStorFmt == SOT_FORMATSTR_ID_STARDRAW_50    ||
		 nStorFmt == SOT_FORMATSTR_ID_STARDRAW_40    ||
		 nStorFmt == SOT_FORMATSTR_ID_STARDRAW )
	{
		StreamMode	eStmMode = STREAM_STD_READWRITE;
		BOOL		bIsEmptyStreams = FALSE;

		if( mbShowProgress && ( mrDocShell.GetCreateMode() != SFX_CREATE_MODE_EMBEDDED ) )
		{
			// create progress
			CreateProgress();

			// !!!
			/*
			if( mpProgress )
				mrDocument.SetIOProgressHdl( *mpProgress );
			*/
		}

		if( pStore->IsStream( pSfxStyleSheets ) )
		{
			SvStorageStreamRef xPoolStm( pStore->OpenStream( pSfxStyleSheets, STREAM_STD_READWRITE ) );

			if( !( bRet = ( xPoolStm.Is() && ( xPoolStm->GetError() == 0 ) ) ) )
			{
				// try to open as read only
				xPoolStm.Clear();
				xPoolStm = pStore->OpenStream( pSfxStyleSheets, eStmMode = STREAM_STD_READ );
				bRet = ( xPoolStm.Is() && xPoolStm->GetError() == 0 );
			}

			if( bRet )
			{
				// load ItemPool
				xPoolStm->SetVersion( pStore->GetVersion() );
				xPoolStm->SetKey( pStore->GetKey() );
				mrDocShell.GetPool().SetFileFormatVersion( (USHORT) pStore->GetVersion() );
				xPoolStm->SetBufferSize( POOL_BUFFER_SIZE );
				mrDocShell.GetPool().Load( *xPoolStm );
				bRet = ( xPoolStm->GetError() == 0 );
				DBG_ASSERT( bRet, "Error while loading ItemPool" );
			}

			if( bRet )
			{
				// load StyleSheet pool
				( (SfxStyleSheetPool*) mrDocShell.GetStyleSheetPool() )->Load( *xPoolStm );
				xPoolStm->SetBufferSize( 0 );
				bRet = ( xPoolStm->GetError() == 0 );
				DBG_ASSERT( bRet, "Error while loading StyleSheetPool" );
			}

			if( !bRet || xPoolStm->GetErrorCode() )
				mrDocShell.SetError( xPoolStm->GetErrorCode() );
		}
		else
		{
			bIsEmptyStreams = TRUE;
			bRet = TRUE;
		}

		if( bRet )
		{
			// load drawing model
			SvStorageStreamRef	xDocStm;
			sal_Bool			bAcceptableStreamFound = sal_False;
			
			if( pStore->IsStream( pStarDrawDoc ) )
			{
				bAcceptableStreamFound = sal_True;
				xDocStm = pStore->OpenStream( pStarDrawDoc, eStmMode );
			}
			else if( pStore->IsStream( pStarDrawDoc3 ))
			{
				bAcceptableStreamFound = sal_True;
				xDocStm = pStore->OpenStream( pStarDrawDoc3, eStmMode );
			}

			if( xDocStm.Is() && bAcceptableStreamFound )
			{
				xDocStm->SetVersion( pStore->GetVersion() );
				xDocStm->SetKey( pStore->GetKey() );

				if( bRet = ( xDocStm->GetError() == 0 ) )
				{
					xDocStm->SetBufferSize( DOCUMENT_BUFFER_SIZE );
					xDocStm->SetKey( pStore->GetKey() );
					mrDocument.SetModelStorage( pStore );
					*xDocStm >> mrDocument;
					mrDocument.SetModelStorage( NULL );

					if( !( bRet = ( xDocStm->GetError() == 0 ) ) )
					{
						if( pStore->GetKey().Len() == 0 )
							mrDocShell.SetError( ERRCODE_SFX_DOLOADFAILED );  // no password? --> file defect
						else
							mrDocShell.SetError( ERRCODE_SFX_WRONGPASSWORD ); // wrong password
					}
					else if ( xDocStm->GetErrorCode() )
						mrDocShell.SetError( xDocStm->GetErrorCode() );

					xDocStm->SetBufferSize( 0 );
				}
				
				bIsEmptyStreams = FALSE;
			}
		}

		if( bRet )
		{
			if( bIsEmptyStreams )
				mrDocument.NewOrLoadCompleted( NEW_DOC );
			else
			{
				mrDocument.NewOrLoadCompleted( DOC_LOADED );
				PostLoadCheckStyleSheetPool( (SfxStyleSheetPool*) mrDocShell.GetStyleSheetPool(), &mrDocument );
			}
		}
	}
	else
		pStore->SetError( SVSTREAM_WRONGVERSION );

	return bRet;
}

// -----------------------------------------------------------------------------

sal_Bool SdBINFilter::Export()
{
	SvtSaveOptions							aOptions;
	SvStorage*								pStore = mrMedium.GetOutputStorage();
	const SvtSaveOptions::SaveGraphicsMode	eSaveMode( aOptions.GetSaveGraphicsMode() );
	const BOOL								bSaveNative = ( SvtSaveOptions::SaveGraphicsOriginal == eSaveMode );
	const BOOL								bSaveCompressed = ( bSaveNative || ( SvtSaveOptions::SaveGraphicsCompressed == eSaveMode ) );
	sal_Bool								bRet = sal_False;

	mrDocument.SetSaveCompressed( bSaveCompressed );
	mrDocument.SetSaveNative( bSaveNative );
	mrDocument.PreSave();

	if( !pStore->GetError() )
	{
		// store ItemPool and StyleSheetPool
		SvStorageStreamRef xPoolStm( pStore->OpenStream( pSfxStyleSheets, STREAM_READ | STREAM_WRITE | STREAM_TRUNC ) );

		if( xPoolStm.Is() && !xPoolStm->GetError() )
		{
			const long		nVersion = pStore->GetVersion();
			const USHORT	nOldComprMode = xPoolStm->GetCompressMode();
			USHORT			nNewComprMode = nOldComprMode;

			xPoolStm->SetVersion( pStore->GetVersion() );
			xPoolStm->SetKey( pStore->GetKey() );
			xPoolStm->SetSize( 0 );
			xPoolStm->SetBufferSize( POOL_BUFFER_SIZE );
			mrDocShell.GetPool().SetFileFormatVersion( (USHORT) nVersion );

			if( SOFFICE_FILEFORMAT_40 <= nVersion )
			{
				if( bSaveNative )
					nNewComprMode |= COMPRESSMODE_NATIVE;

				if( bSaveCompressed )
					nNewComprMode |= COMPRESSMODE_ZBITMAP;

				xPoolStm->SetCompressMode( nNewComprMode );
			}

			mrDocShell.GetPool().Store( *xPoolStm );
			mrDocShell.GetStyleSheetPool()->SetSearchMask( SFX_STYLE_FAMILY_ALL );
			mrDocShell.GetStyleSheetPool()->Store( *xPoolStm, FALSE );
			xPoolStm->SetBufferSize( 0 );
			xPoolStm->SetCompressMode( nOldComprMode );
			bRet = ( xPoolStm->GetError() == 0 );
		}
		else 
		{
			DBG_ERROR( "Error while writing the pool" );
			bRet = sal_False;
		}

		if( !bRet || xPoolStm->GetErrorCode() )
			mrDocShell.SetError( xPoolStm->GetErrorCode() );
	}

	if( bRet && ( mrDocShell.GetCreateMode() != SFX_CREATE_MODE_ORGANIZER ) )
	{
		// rename stream is neccessary
		if( pStore->IsStream( pStarDrawDoc ) && !pStore->Rename( pStarDrawDoc, pStarDrawDoc3 ) )
		{
			DBG_ERROR( "Error while renaming the stream");
		}

		// store model
		SvStorageStreamRef xDocStm( pStore->OpenStream( pStarDrawDoc3, STREAM_READ | STREAM_WRITE | STREAM_TRUNC ) );

		if( xDocStm.Is() && !xDocStm->GetError() )
		{
			xDocStm->SetVersion( pStore->GetVersion() );
			xDocStm->SetKey( pStore->GetKey() );
			xDocStm->SetSize( 0 );
			xDocStm->SetBufferSize( DOCUMENT_BUFFER_SIZE );
			xDocStm->SetKey( pStore->GetKey() );
			*xDocStm << mrDocument;
			xDocStm->SetBufferSize( 0 );
			bRet = ( xDocStm->GetError() == 0 );
		}
		else
		{
			DBG_ERROR( "Error while writing the model" );
			bRet = sal_False;
		}
		
		if( !bRet || xDocStm->GetErrorCode() )
			mrDocShell.SetError( xDocStm->GetErrorCode() );
	}

/* !!!
	if( bRet && ( mrDocShell.GetCreateMode() != SFX_CREATE_MODE_EMBEDDED ) )
		mrDocShell.AddXMLAsZipToTheStorage( *pStore );
*/

	mrDocument.PostSave();

	return bRet;
}
