/*************************************************************************
 *
 *  $RCSfile: namecont.hxx,v $
 *
 *  $Revision: 1.8 $
 *
 *  last change: $Author: ab $ $Date: 2001/07/09 16:04:43 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SFX_NAMECONT_HXX
#define _SFX_NAMECONT_HXX

#include <hash_map>


#ifndef _COM_SUN_STAR_SCRIPT_XLIBRARYCONTAINER_HPP_
#include <com/sun/star/script/XLibraryContainer.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XNAMECONTAINER_HPP_
#include <com/sun/star/container/XNameContainer.hpp>
#endif
#ifndef _COM_SUN_STAR_CONTAINER_XCONTAINER_HPP_
#include <com/sun/star/container/XContainer.hpp>
#endif
#ifndef _COM_SUN_STAR_IO_XOUTPUTSTREAM_HPP_
#include <com/sun/star/io/XOutputStream.hpp>
#endif

#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif
#ifndef _CPPUHELPER_COMPONENT_HXX_
#include <cppuhelper/component.hxx>
#endif
#include <cppuhelper/typeprovider.hxx>
#include <cppuhelper/interfacecontainer.hxx>


#ifndef _SOT_STORAGE_HXX
#include <sot/storage.hxx>
#endif

#include <cppuhelper/implbase2.hxx>
typedef ::cppu::WeakImplHelper2< ::com::sun::star::script::XLibraryContainer,
	 ::com::sun::star::container::XContainer > LibraryContainerHelper;


namespace SfxContainer_Impl
{

typedef ::cppu::WeakImplHelper2< ::com::sun::star::container::XNameContainer,
	::com::sun::star::container::XContainer > NameContainerHelper;


struct hashName_Impl
{
	size_t operator()(const ::rtl::OUString Str) const
	{
		return (size_t)Str.hashCode();
	}
};

struct eqName_Impl
{
	sal_Bool operator()(const ::rtl::OUString Str1, const ::rtl::OUString Str2) const
	{
		return ( Str1 == Str2 );
	}
};

typedef std::hash_map
<
	::rtl::OUString,
	sal_Int32,
	hashName_Impl,
	eqName_Impl
>
NameContainerNameMap;



//============================================================================

struct MutexHolder
{
	::osl::Mutex mMutex;
};

class NameContainer_Impl : public MutexHolder, public NameContainerHelper
{
	NameContainerNameMap mHashMap;
	::com::sun::star::uno::Sequence< ::rtl::OUString > mNames;
	::com::sun::star::uno::Sequence< ::com::sun::star::uno::Any > mValues;
	sal_Int32 mnElementCount;

	::com::sun::star::uno::Type mType;
	::com::sun::star::uno::XInterface* mpxEventSource;

	::cppu::OInterfaceContainerHelper maListenerContainer;

public:
	NameContainer_Impl( const ::com::sun::star::uno::Type& rType )
		: mType( rType )
        , mpxEventSource( NULL )
		, mnElementCount( 0 )
		, maListenerContainer( mMutex )
	{}

	void setEventSource( ::com::sun::star::uno::XInterface* pxEventSource )
		{ mpxEventSource = pxEventSource; }

    // Methods XElementAccess
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) 
		throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasElements(  ) 
		throw(::com::sun::star::uno::RuntimeException);

    // Methods XNameAccess
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) 
		throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) 
		throw(::com::sun::star::uno::RuntimeException);

    // Methods XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& aName, const ::com::sun::star::uno::Any& aElement ) 
		throw(::com::sun::star::lang::IllegalArgumentException, 
			  ::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);

    // Methods XNameContainer
    virtual void SAL_CALL insertByName( const ::rtl::OUString& aName, const ::com::sun::star::uno::Any& aElement )
		throw(::com::sun::star::lang::IllegalArgumentException, 
			  ::com::sun::star::container::ElementExistException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL removeByName( const ::rtl::OUString& Name ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);

	// Methods XContainer
	virtual void SAL_CALL addContainerListener( const ::com::sun::star::uno::Reference< 
		::com::sun::star::container::XContainerListener >& xListener ) 
			throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL removeContainerListener( const ::com::sun::star::uno::Reference< 
		::com::sun::star::container::XContainerListener >& xListener ) 
			throw (::com::sun::star::uno::RuntimeException);

};

}	// namespace SfxContainer_Impl


//============================================================================

class SfxLibrary_Impl;

class SfxLibraryContainer_Impl : public LibraryContainerHelper
{
	SfxContainer_Impl::NameContainer_Impl maNameContainer;
	sal_Bool mbModified;

	::rtl::OUString maLibInfoFileName;
	::rtl::OUString maLibElementFileExtension;
	::rtl::OUString maLibraryPath;
	::rtl::OUString maLibrariesDir;

	SotStorageRef mxStorage;

	void implStoreLibrary( SfxLibrary_Impl* pLib, const ::rtl::OUString& aName, SotStorageRef xStorage );

	// Methods to distinguish between deffirent library types
	virtual SfxLibrary_Impl* SAL_CALL implCreateLibrary( void ) = 0;
    virtual SfxLibrary_Impl* SAL_CALL implCreateLibraryLink
		( const ::rtl::OUString& StorageURL, sal_Bool ReadOnly ) = 0;
	virtual ::com::sun::star::uno::Any SAL_CALL createEmptyLibraryElement( void ) = 0;
	virtual sal_Bool SAL_CALL isLibraryElementValid( ::com::sun::star::uno::Any aElement ) = 0;
	virtual void SAL_CALL writeLibraryElement
	( 
		::com::sun::star::uno::Any aElement,
		const ::rtl::OUString& aElementName, 
		::com::sun::star::uno::Reference< ::com::sun::star::io::XOutputStream > xOutput 
	)
		throw(::com::sun::star::uno::Exception) = 0;
	virtual ::com::sun::star::uno::Any SAL_CALL importLibraryElement
		( const ::rtl::OUString& aFile, SotStorageStreamRef xElementStream ) = 0;

public:
	SfxLibraryContainer_Impl( const ::rtl::OUString& aLibInfoFileName,
							  const ::rtl::OUString& aLibElementFileExtension,
							  const ::rtl::OUString& aLibraryPath,
							  const ::rtl::OUString& aLibrariesDir );
	sal_Bool init( SotStorageRef xStorage );

	// Interface to release and reset storage for saving
	void setStorage( SotStorageRef xStorage )
	{ 
		mxStorage = xStorage; 
	}

	// TODO: Methods of new XLibraryStorage interface?
	virtual void SAL_CALL storeLibraries( sal_Bool bComplete );
	virtual void SAL_CALL storeLibrariesToStorage( SotStorageRef xStorage );
	virtual void SAL_CALL storeLibraries_Impl( SotStorageRef xStorage, sal_Bool bComplete );

    // Methods XElementAccess
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType()
		throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasElements() 
		throw(::com::sun::star::uno::RuntimeException);

    // Methods XNameAccess
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames()
		throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) 
		throw(::com::sun::star::uno::RuntimeException);

    // Methods XLibraryContainer
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::container::XNameContainer > SAL_CALL 
		createLibrary( const ::rtl::OUString& Name ) 
			throw(::com::sun::star::lang::IllegalArgumentException, 
				  ::com::sun::star::container::ElementExistException, 
				  ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Reference< ::com::sun::star::container::XNameAccess > SAL_CALL createLibraryLink
		( const ::rtl::OUString& Name, const ::rtl::OUString& StorageURL, sal_Bool ReadOnly ) 
			throw(::com::sun::star::lang::IllegalArgumentException, 
				  ::com::sun::star::container::ElementExistException, 
				  ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL removeLibrary( const ::rtl::OUString& Name ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL isLibraryLoaded( const ::rtl::OUString& Name ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL loadLibrary( const ::rtl::OUString& Name ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);

	// Methods XContainer
	virtual void SAL_CALL addContainerListener( const ::com::sun::star::uno::Reference< 
		::com::sun::star::container::XContainerListener >& xListener ) 
			throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL removeContainerListener( const ::com::sun::star::uno::Reference< 
		::com::sun::star::container::XContainerListener >& xListener ) 
			throw (::com::sun::star::uno::RuntimeException);
};


//============================================================================

struct OLibraryMutex
{
	::osl::Mutex m_mutex;
};

class SfxLibrary_Impl
	: public ::com::sun::star::container::XNameContainer
	, public ::com::sun::star::container::XContainer
	, public OLibraryMutex
	, public ::cppu::OComponentHelper
{
	friend class SfxLibraryContainer_Impl;
	friend class SfxScriptLibraryContainer;

	SfxContainer_Impl::NameContainer_Impl maNameContainer;
	//OInterfaceContainerHelper maListenerContainer;

	sal_Bool mbLoaded;
	sal_Bool mbModified;

	// Data for linked library
	::rtl::OUString maStorageURL;
	sal_Bool mbReadOnly;


public:
	SfxLibrary_Impl( ::com::sun::star::uno::Type aType );
	SfxLibrary_Impl( ::com::sun::star::uno::Type aType,
		const ::rtl::OUString& aStorageURL, sal_Bool ReadOnly );


	// Methods XInterface
    virtual ::com::sun::star::uno::Any SAL_CALL queryInterface( const ::com::sun::star::uno::Type& rType ) throw( ::com::sun::star::uno::RuntimeException );
	virtual void SAL_CALL acquire() throw() { OComponentHelper::acquire(); }
	virtual void SAL_CALL release() throw() { OComponentHelper::release(); }

    // Methods XElementAccess
    virtual ::com::sun::star::uno::Type SAL_CALL getElementType(  ) 
		throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasElements(  ) 
		throw(::com::sun::star::uno::RuntimeException);

    // Methods XNameAccess
    virtual ::com::sun::star::uno::Any SAL_CALL getByName( const ::rtl::OUString& aName ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual ::com::sun::star::uno::Sequence< ::rtl::OUString > SAL_CALL getElementNames(  ) 
		throw(::com::sun::star::uno::RuntimeException);
    virtual sal_Bool SAL_CALL hasByName( const ::rtl::OUString& aName ) 
		throw(::com::sun::star::uno::RuntimeException);

    // Methods XNameReplace
    virtual void SAL_CALL replaceByName( const ::rtl::OUString& aName, const ::com::sun::star::uno::Any& aElement ) 
		throw(::com::sun::star::lang::IllegalArgumentException, 
			  ::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);

    // Methods XNameContainer
    virtual void SAL_CALL insertByName( const ::rtl::OUString& aName, const ::com::sun::star::uno::Any& aElement )
		throw(::com::sun::star::lang::IllegalArgumentException, 
			  ::com::sun::star::container::ElementExistException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);
    virtual void SAL_CALL removeByName( const ::rtl::OUString& Name ) 
		throw(::com::sun::star::container::NoSuchElementException, 
			  ::com::sun::star::lang::WrappedTargetException, 
			  ::com::sun::star::uno::RuntimeException);

	// XTypeProvider
    ::com::sun::star::uno::Sequence< ::com::sun::star::uno::Type > SAL_CALL getTypes(  ) 
		throw( ::com::sun::star::uno::RuntimeException );
    ::com::sun::star::uno::Sequence<sal_Int8> SAL_CALL getImplementationId(  ) 
		throw( ::com::sun::star::uno::RuntimeException );

	// Methods XContainer
	virtual void SAL_CALL addContainerListener( const ::com::sun::star::uno::Reference< 
		::com::sun::star::container::XContainerListener >& xListener ) 
			throw (::com::sun::star::uno::RuntimeException);
	virtual void SAL_CALL removeContainerListener( const ::com::sun::star::uno::Reference< 
		::com::sun::star::container::XContainerListener >& xListener ) 
			throw (::com::sun::star::uno::RuntimeException);
};


#endif

