/*************************************************************************
 *
 *  $RCSfile: frmview.cxx,v $
 *
 *  $Revision: 1.2 $
 *
 *  last change: $Author: mh $ $Date: 2000/10/23 16:22:19 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef _SVXIDS_HRC
#include <svx/svxids.hrc>
#endif

#include "frmview.hxx"
#include <sd/drawdoc.hxx>
#include "sdresid.hxx"
#include <sd/pres.hxx>
#include <sd/glob.hrc>
#include <sd/sdiocmpt.hxx>


/*************************************************************************
|*
|* Ctor
|*
\************************************************************************/

FrameView::FrameView(SdDrawDocument* pDrawDoc)
  : SdrView(pDrawDoc, (OutputDevice*) NULL),
  nRefCount(0),
  nPresViewShellId(SID_VIEWSHELL0),
  nSlotId(SID_OBJECT_SELECT)
{
}

/*************************************************************************
|*
|* Dtor
|*
\************************************************************************/

FrameView::~FrameView()
{
}


/*************************************************************************
|*
|* Verbindung herstellen
|*
\************************************************************************/

void FrameView::Connect()
{
	nRefCount++;
}


/*************************************************************************
|*
|* Verbindung loesen
|*
\************************************************************************/

void FrameView::Disconnect()
{
	if (nRefCount > 0)
	{
		nRefCount--;
	}

	if (nRefCount == 0)
	{
		delete this;
	}
}


/*************************************************************************
|*
|* Inserter fuer SvStream zum Speichern
|*
\************************************************************************/

SvStream& operator << (SvStream& rOut, const FrameView& rView)
{
	return rOut;
}

/*************************************************************************
|*
|* Extractor fuer SvStream zum Laden
|*
\************************************************************************/

SvStream& operator >> (SvStream& rIn, FrameView& rView)
{
	rIn >> (SdrView&) rView;

	SdIOCompat aIO(rIn, STREAM_READ);

	rIn >> rView.bRuler;
	rIn >> rView.aVisibleLayers;
	rIn >> rView.aLockedLayers;
	rIn >> rView.aPrintableLayers;
	rIn >> rView.aStandardHelpLines;

	if (aIO.GetVersion() >= 1)
	{
		// Daten der Versionen >= 1 einlesen
		rIn >> rView.aNotesHelpLines;
		rIn >> rView.aHandoutHelpLines;
	}

	if (aIO.GetVersion() >= 2)
	{
		// Daten der Versionen >= 2 einlesen
		rIn >> rView.bNoColors;
		rIn >> rView.bNoAttribs;
	}

	if (aIO.GetVersion() >= 3)
	{
		ULONG nULTemp;
		rIn >> rView.aVisArea;
        rIn >> nULTemp;          rView.ePageKind = (PageKind) nULTemp;
		rIn >> rView.nSelectedPage;
        rIn >> nULTemp;          rView.eStandardEditMode = (EditMode) nULTemp;
        rView.eNotesEditMode   = rView.eStandardEditMode;
        rView.eHandoutEditMode = rView.eStandardEditMode;
		rIn >> rView.bLayerMode;
	}

	if (aIO.GetVersion() >= 4)
	{
		rIn >> rView.bQuickEdit;
	}

	if (aIO.GetVersion() >= 5)
	{
		rIn >> rView.bDragWithCopy;
	}

    if (aIO.GetVersion() >= 6)
	{
        UINT16 nTemp;
        rIn >> nTemp; rView.nSlidesPerRow = (USHORT)nTemp;
	}

    if (aIO.GetVersion() >= 7)
	{
		rIn >> rView.bBigHandles;
		rIn >> rView.bDoubleClickTextEdit;
		rIn >> rView.bClickChangeRotation;
	}

    if (aIO.GetVersion() >= 8)
	{
        ULONG nULTemp;
        rIn >> nULTemp; rView.eNotesEditMode   = (EditMode) nULTemp;
        rIn >> nULTemp; rView.eHandoutEditMode = (EditMode) nULTemp;
    }

	if (aIO.GetVersion() >= 9)
	{
		rIn >> rView.nDrawMode;
		rIn >> rView.nPreviewDrawMode;
	}

	if (aIO.GetVersion() >= 10)
	{
		rIn >> rView.bShowPreviewInPageMode;
		rIn >> rView.bShowPreviewInMasterPageMode;
	}

	if (aIO.GetVersion() >= 11)
	{
		rIn >> rView.bShowPreviewInOutlineMode;
	}

    // Falls die UniqueLayerNames vorhanden sind, werden die Default-Namen
    // verwendet
    String aLayerName(rView.GetActiveLayer());

    if (aLayerName.CompareToAscii("LAYER_LAYOUT") == COMPARE_EQUAL )
    {
        rView.SetActiveLayer(String(SdResId(STR_LAYER_LAYOUT)));
    }
    else if (aLayerName.CompareToAscii("LAYER_BCKGRND") == COMPARE_EQUAL )
    {
        rView.SetActiveLayer(String(SdResId(STR_LAYER_BCKGRND)));
    }
    else if (aLayerName.CompareToAscii("LAYER_BACKGRNDOBJ") == COMPARE_EQUAL )
    {
        rView.SetActiveLayer(String(SdResId(STR_LAYER_BCKGRNDOBJ)));
    }
    else if (aLayerName.CompareToAscii("LAYER_CONTROLS") == COMPARE_EQUAL )
    {
        rView.SetActiveLayer(String(SdResId(STR_LAYER_CONTROLS)));
    }
    else if (aLayerName.CompareToAscii("LAYER_MEASURELINES") == COMPARE_EQUAL )
    {
        rView.SetActiveLayer(String(SdResId(STR_LAYER_MEASURELINES)));
    }

	if (rView.GetModel())
	{
	    USHORT nMaxPages = ((SdDrawDocument* )rView.GetModel())->
                                      GetSdPageCount(rView.ePageKind);

		if (rView.nSelectedPage >= nMaxPages)
		{
			// Ggf. auf die letzte Seite selektieren
   	 		rView.nSelectedPage = nMaxPages - 1;
		}
	}

	return rIn;
}

/*************************************************************************
|*
|* Update mit Daten der SdOptions
|*
\************************************************************************/

void FrameView::Update(SdOptions* pOptions)
{
}


/*************************************************************************
|*
|* EditMode (Page oder MasterPage) des Arbeitsmodus setzen
|*
\************************************************************************/

void FrameView::SetViewShEditMode(EditMode eMode, PageKind eKind)
{
    if (eKind == PK_STANDARD)
    {
        eStandardEditMode = eMode;
    }
    else if (eKind == PK_NOTES)
    {
        eNotesEditMode = eMode;
    }
    else if (eKind == PK_HANDOUT)
    {
        eHandoutEditMode = eMode;
    }
}


/*************************************************************************
|*
|* EditMode (Page oder MasterPage) des Arbeitsmodus zurueckgeben
|*
\************************************************************************/

EditMode FrameView::GetViewShEditMode(PageKind eKind)
{
    EditMode eMode = EM_PAGE;

    if (eKind == PK_STANDARD)
    {
        eMode = eStandardEditMode;
    }
    else if (eKind == PK_NOTES)
    {
        eMode = eNotesEditMode;
    }
    else if (eKind == PK_HANDOUT)
    {
        eMode = eHandoutEditMode;
    }

    return (eMode);
}


