/*************************************************************************
 *
 *  $RCSfile: showwin.cxx,v $
 *
 *  $Revision: 1.7 $
 *
 *  last change: $Author: thb $ $Date: 2001/07/11 15:10:42 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/

#ifndef INCLUDED_SVTOOLS_SYSLOCALE_HXX
#include <svtools/syslocale.hxx>
#endif
#ifndef _E3D_VIEW3D_HXX //autogen wg. E3dView
#include <svx/view3d.hxx>
#endif

#include "sip.hrc"
#include "showwin.hxx"
#include "viewshel.hxx"

#include "sdresid.hxx"
#include "fuslshow.hxx"
#include "sdresid.hxx"
#include "strings.hrc"
#include "view.hxx"
#include "hid.hrc"

#define SHOWWIN_FONTHEIGHT 14

/*************************************************************************
|*
|* Konstruktor
|*
\************************************************************************/

SdShowWindow::SdShowWindow( Window* pParent, FuSlideShow* _pFuSlideShow ) :
	Window(pParent,WB_CLIPCHILDREN),
	pFuSlideShow(_pFuSlideShow),
	bEndMode(FALSE),
	bPauseMode(FALSE),
	nRestartPage( PAGE_NO_END ),
	nPauseTimeout( SLIDE_NO_TIMEOUT ),
	pViewShell(NULL)
{	
	MapMode aMap(GetMapMode());
	aMap.SetMapUnit(MAP_100TH_MM);
	SetMapMode(aMap);

	SetHelpId(HID_SIP_SHOWWIN);

	aPauseTimer.SetTimeoutHdl( LINK( this, SdShowWindow, PauseTimeoutHdl ) );
	aPauseTimer.SetTimeout( 1000 );

	Bitmap 	  aBitmap(SdResId(BMP_WAIT_ICON));
	BitmapEx  aBmpEx( aBitmap, COL_LIGHTMAGENTA );
	AlphaMask aMask( aBmpEx.GetMask() );
	aMask.Replace( 0, 128 );
	aWaitIcon = BitmapEx( aBmpEx.GetBitmap(), aMask );

	aRestoreSpace.SetSizePixel( aWaitIcon.GetSizePixel() );
}

/*************************************************************************
|*
|* Destruktor
|*
\************************************************************************/

SdShowWindow::~SdShowWindow()
{
}

/*************************************************************************
|*
|* Keyboard event
|*
\************************************************************************/

void SdShowWindow::KeyInput(const KeyEvent& rKEvt)
{
	if( pViewShell && pViewShell->isLocked() )
	{
		pViewShell->Unlock();
		return;
	}

	if( bEndMode )
		TerminateShow();
	else if( bPauseMode )
	{
		if( rKEvt.GetKeyCode().GetCode() == KEY_ESCAPE )
			TerminateShow();
		else
			RestartShow();
	}
	else
	{
		BOOL bReturn = FALSE;

		if(pViewShell)
			bReturn = pViewShell->KeyInput(rKEvt, this);

		if( !bReturn )
			Window::KeyInput(rKEvt);
	}
}

/*************************************************************************
|*
|* MouseButtonDown event
|*
\************************************************************************/

void __EXPORT SdShowWindow::MouseButtonDown(const MouseEvent& rMEvt)
{
	if( pViewShell && pViewShell->isLocked() )
	{
		pViewShell->Unlock();
		return;
	}

	if( !bEndMode && !bPauseMode )
	{
		if(pViewShell)
			pViewShell->MouseButtonDown(rMEvt, this);
	}
}

/*************************************************************************
|*
|* MouseMove event
|*
\************************************************************************/

void __EXPORT SdShowWindow::MouseMove(const MouseEvent& rMEvt)
{
	if( pViewShell )
	{
		if( !pViewShell->isLocked() )
		{
			if( !bEndMode && !bPauseMode )
				pViewShell->MouseMove(rMEvt, this);
		}
	}
}

/*************************************************************************
|*
|* MouseButtonUp event
|*
\************************************************************************/

void __EXPORT SdShowWindow::MouseButtonUp(const MouseEvent& rMEvt)
{
	if( pViewShell && pViewShell->isLocked() )
	{
		pViewShell->Unlock();
		return;
	}

	if( bEndMode )
		TerminateShow();
	else if( bPauseMode )
		RestartShow();
	else if(pViewShell)
		pViewShell->MouseButtonUp(rMEvt,this);
}

/*************************************************************************
|*
|* Paint-Event: wenn FuSlideShow noch erreichbar ist, weiterleiten
|*
\************************************************************************/

void __EXPORT SdShowWindow::Paint(const Rectangle& rRect)
{
	if( bEndMode )
		DrawEndScene();
	else if( bPauseMode )
		DrawPauseScene( FALSE );
	else if (pViewShell)
		pViewShell->Paint(rRect, this);
}

/*************************************************************************
|*
|* ParentNotify
|*
\************************************************************************/

long __EXPORT SdShowWindow::ParentNotify(NotifyEvent& rNEvt)
{
    long nOK = FALSE;

//  if( pViewShell && rNEvt.GetType() == EVENT_GETFOCUS )
//    {
//        NotifyEvent aNEvt(EVENT_GETFOCUS, this);
//TODO:FRame?        nOK = pViewShell->GetViewFrame()->GetWindow().ParentNotify(aNEvt);
//    }

    if (!nOK)
        nOK = Window::Notify(rNEvt);

    return nOK;
}


/*************************************************************************
|*
|* GetFocus()
|* Window des MDIFrames benachrichtigen
|*
\************************************************************************/

void SdShowWindow::GetFocus()
{
    // Basisklasse
    Window::GetFocus();

	//TODO:???
/*
    if( pViewShell )
    {
        NotifyEvent aNEvt(EVENT_GETFOCUS, this);
        pViewShell->GetViewFrame()->GetWindow().ParentNotify(aNEvt);
    }
*/
}

void SdShowWindow::LoseFocus()
{
	BOOL bStop = TRUE;
}

/*************************************************************************
|*
|* GetFocus()
|* Window des MDIFrames benachrichtigen
|*
\************************************************************************/

BOOL SdShowWindow::SetEndMode()
{
	if( !bPauseMode && !bEndMode && pViewShell && pViewShell->GetView() )
	{
		pViewShell->GetView()->DelWin( this );
		bEndMode = TRUE;
		aShowBackground = GetBackground();
		SetBackground( Wallpaper( Color( COL_BLACK ) ) );

		Invalidate();
	}
	return bEndMode;
}

// -----------------------------------------------------------------------------

BOOL SdShowWindow::SetPauseMode( USHORT nPageToRestart, ULONG nTimeout, Graphic* pLogo )
{
	if( pViewShell && pViewShell->GetSlideShow() && !nTimeout )
	{
		pViewShell->GetSlideShow()->JumpToPage( nPageToRestart );
	}
	else if( !bPauseMode && !bEndMode && pViewShell && pViewShell->GetView() )
	{
		pViewShell->GetView()->DelWin( this );
		nPauseTimeout = nTimeout;
		nRestartPage = nPageToRestart; 
		bPauseMode = TRUE;
		aShowBackground = GetBackground();
		SetBackground( Wallpaper( Color( COL_BLACK ) ) );

		if( pLogo )
			aLogo = *pLogo;

		Invalidate();

		if( SLIDE_NO_TIMEOUT != nPauseTimeout )
			aPauseTimer.Start();
	}
	return bPauseMode;
}

// -----------------------------------------------------------------------------

void SdShowWindow::TerminateShow()
{
	aLogo.Clear();
	aPauseTimer.Stop();
	Erase();
	SetBackground( aShowBackground );
	bEndMode = bPauseMode = FALSE;
	nPauseTimeout = SLIDE_NO_TIMEOUT;

	if( pViewShell && pViewShell->GetSlideShow() )
	{
		if( pViewShell->GetView() )
			pViewShell->GetView()->AddWin( this );

		pViewShell->GetSlideShow()->Terminate();
	}
	nRestartPage = PAGE_NO_END;
}

// -----------------------------------------------------------------------------

void SdShowWindow::RestartShow()
{
	aLogo.Clear();
	aPauseTimer.Stop();
	Erase();
	SetBackground( aShowBackground );
	bEndMode = bPauseMode = FALSE;
	nPauseTimeout = SLIDE_NO_TIMEOUT;

	if( pViewShell && pViewShell->GetSlideShow() )
	{
		if( pViewShell->GetView() )
			pViewShell->GetView()->AddWin( this );

		pViewShell->GetSlideShow()->JumpToPage( nRestartPage );
	}
	nRestartPage = PAGE_NO_END;
}

// -----------------------------------------------------------------------------

void SdShowWindow::DrawPauseScene( BOOL bTimeoutOnly )
{
	const MapMode&	rMap = GetMapMode();
	const Point		aOutOrg( PixelToLogic( Point() ) );
	const Size		aOutSize( GetOutputSize() );
	const Size		aTextSize( LogicToLogic( Size( 0, SHOWWIN_FONTHEIGHT ), MAP_POINT, rMap ) );
	const Size		aOffset( LogicToLogic( Size( 1000, 1000 ), MAP_100TH_MM, rMap ) );
	String			aText( SdResId( STR_PRES_PAUSE ) );
	BOOL			bDrawn = FALSE;

	Font			aFont;
	const Font 		aOldFont( GetFont() );

	aFont.SetSize( aTextSize );
	aFont.SetColor( COL_WHITE );
	aFont.SetCharSet( aOldFont.GetCharSet() );
	aFont.SetLanguage( aOldFont.GetLanguage() );	

	if( !bTimeoutOnly && ( aLogo.GetType() != GRAPHIC_NONE ) )
	{
		Size aGrfSize;

		if( aLogo.GetPrefMapMode() == MAP_PIXEL )
			aGrfSize = PixelToLogic( aLogo.GetPrefSize() );
		else
			aGrfSize = LogicToLogic( aLogo.GetPrefSize(), aLogo.GetPrefMapMode(), rMap );

		const Point	aGrfPos( Max( aOutOrg.X() + aOutSize.Width() - aGrfSize.Width() - aOffset.Width(), aOutOrg.X() ),
							 Max( aOutOrg.Y() + aOutSize.Height() - aGrfSize.Height() - aOffset.Height(), aOutOrg.Y() ) );

		if( aLogo.IsAnimated() )
			aLogo.StartAnimation( this, aGrfPos, aGrfSize, (long) this );
		else
			aLogo.Draw( this, aGrfPos, aGrfSize );
	}

	if( SLIDE_NO_TIMEOUT != nPauseTimeout )
	{
		MapMode			aVMap( rMap );
		VirtualDevice	aVDev( *this );

		aVMap.SetOrigin( Point() );
		aVDev.SetMapMode( aVMap );
		aVDev.SetBackground( Wallpaper( Color( COL_BLACK ) ) );

		// set font first, to determine real output height
		aVDev.SetFont( aFont );
			
		const Size aVDevSize( aOutSize.Width(), aVDev.GetTextHeight() );

		if( aVDev.SetOutputSize( aVDevSize ) )
		{
			// Note: if performance gets an issue here, we can use NumberFormatter directly
			SvtSysLocale				aSysLocale;
			const LocaleDataWrapper&    aLocaleData = aSysLocale.GetLocaleData();
			
			aText.AppendAscii( RTL_CONSTASCII_STRINGPARAM( " ( " ));
			aText += aLocaleData.getDuration( Time( 0, 0, nPauseTimeout ) );
			aText.AppendAscii( RTL_CONSTASCII_STRINGPARAM( " )" ));
			aVDev.DrawText( Point( aOffset.Width(), 0 ), aText );
			DrawOutDev( Point( aOutOrg.X(), aOffset.Height() ), aVDevSize, Point(), aVDevSize, aVDev );
			bDrawn = TRUE;
		}
	}

	if( !bDrawn )
	{
		SetFont( aFont );
		DrawText( Point( aOutOrg.X() + aOffset.Width(), aOutOrg.Y() + aOffset.Height() ), aText );
		SetFont( aOldFont );
	}
}

// -----------------------------------------------------------------------------

void SdShowWindow::DrawEndScene()
{
	const Font		aOldFont( GetFont() );
	Font			aFont;
	const Point		aOutOrg( PixelToLogic( Point() ) );
	const Size		aTextSize( LogicToLogic( Size( 0, SHOWWIN_FONTHEIGHT ), MAP_POINT, GetMapMode() ) );
	const String	aText( SdResId( STR_PRES_SOFTEND ) );

	aFont.SetSize( aTextSize );
	aFont.SetColor( COL_WHITE );
	aFont.SetCharSet( aOldFont.GetCharSet() );
	aFont.SetLanguage( aOldFont.GetLanguage() );
	SetFont( aFont );
	DrawText( Point( aOutOrg.X() + aTextSize.Height(), aOutOrg.Y() + aTextSize.Height() ), aText );
	SetFont( aOldFont );
}

// -----------------------------------------------------------------------------

IMPL_LINK( SdShowWindow, PauseTimeoutHdl, Timer*, pTimer )
{
	if( !( --nPauseTimeout ) )
		RestartShow();
	else
	{
		DrawPauseScene( TRUE );
		pTimer->Start();
	}

	return 0L;
}

// -----------------------------------------------------------------------------

void SdShowWindow::Resize()
{
	Window::Resize();
	if(pViewShell)
		pViewShell->Resize();
}

// -----------------------------------------------------------------------------

const int BORDER = 16;

void SdShowWindow::DrawWaitIcon()
{
	const MapMode nOldMode = GetMapMode();
	SetMapMode( MapMode(MAP_PIXEL) );

	Size aOutputSize = GetOutputSizePixel();
	Size aIconSize = aWaitIcon.GetSizePixel();

	Point aPoint( BORDER, aOutputSize.Height() - aIconSize.Height() - BORDER );

	aRestoreSpace = GetBitmap( aPoint, aIconSize );

	DrawBitmapEx( aPoint, aWaitIcon );

	SetMapMode( nOldMode );
}

// -----------------------------------------------------------------------------

void SdShowWindow::ClearWaitIcon()
{
	const MapMode nOldMode = GetMapMode();
	SetMapMode( MapMode(MAP_PIXEL) );

	Size aOutputSize = GetOutputSizePixel();
	Size aIconSize = aWaitIcon.GetSizePixel();

	Point aPoint( BORDER, aOutputSize.Height() - aIconSize.Height() - BORDER );

	DrawBitmap( aPoint, aRestoreSpace );

	SetMapMode( nOldMode );
}

