/*************************************************************************
 *
 *  $RCSfile: viewoptions.hxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: mba $ $Date: 2000/12/08 10:58:40 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#ifndef INCLUDED_SVTOOLS_VIEWOPTIONS_HXX
#define INCLUDED_SVTOOLS_VIEWOPTIONS_HXX

//_________________________________________________________________________________________________________________
//	includes
//_________________________________________________________________________________________________________________

#ifndef _UTL_CONFIGMGR_HXX_
#include <unotools/configmgr.hxx>
#endif

#ifndef _UTL_CONFIGITEM_HXX_
#include <unotools/configitem.hxx>
#endif

#ifndef _SAL_TYPES_H_
#include <sal/types.h>
#endif

#ifndef _OSL_MUTEX_HXX_
#include <osl/mutex.hxx>
#endif

#ifndef _RTL_USTRING_
#include <rtl/ustring>
#endif

#ifndef _COM_SUN_STAR_UNO_SEQUENCE_H_
#include <com/sun/star/uno/Sequence.h>
#endif

//_________________________________________________________________________________________________________________
//	forward declarations
//_________________________________________________________________________________________________________________

class SvtViewDialogOptions_Impl		;
class SvtViewTabDialogOptions_Impl	;
class SvtViewTabPageOptions_Impl	;
class SvtViewWindowOptions_Impl		;

//_________________________________________________________________________________________________________________
//	declarations
//_________________________________________________________________________________________________________________

/*-************************************************************************************************************//**
	@descr			Use these enum values to specify right list in configuration in which your view data are saved.
*//*-*************************************************************************************************************/

enum EViewType
{
	E_DIALOG	=	0,
	E_TABDIALOG	=	1,
	E_TABPAGE	=	2,
	E_WINDOW	=	3
};

/*-************************************************************************************************************//**
	@short			collect informations about view features
	@descr          We support different basetypes of views like dialogs, tab-dialogs, tab-pages and normal windows.
					You must specify your basetype by using right enum value and must give us a valid name for your
					subkey in registry! We support some fix features for some bastypes and user data as string for all!
					see also configuration package "org.openoffice.Office.Views/..." for further informations.

					template of configuration:
						DialogType
							/X			[int]
							/Y			[int]
							/Width		[int]
							/Height		[int]
							/UserData	[string]
						TabDialogType
							/X			[int]
							/Y			[int]
							/PageID		[int]
							/UserData	[string]
						TabPageType
							/UserData	[string]
						WindowType
							/X			[int]
							/Y			[int]
							/Width		[int]
							/Height		[int]
							/Visible	[boolean]
							/UserData	[string]

					structure of configuration:

						org.openoffice.Office.Views	[package]
													/Dialogs	[set]
																/Dialog_FileOpen			[DialogType]
																/Dialog_ImportGraphics		[DialogType]
																...
																/Dialog_<YourName>			[DialogType]

													/TabDialogs	[set]
																/TabDialog_001				[TabDialogType]
																/TabDialog_Blubber			[TabDialogType]
																...
																/TabDialog_<YourName>		[TabDialogType]

													/TabPages	[set]
																/TabPage_XXX				[TabPageType]
																/TabPage_Date				[TabPageType]
																...
																/TabPage_<YourName>			[TabPageType]

													/Windows	[set]
																/Window_User				[WindowType]
																/Window_Options				[WindowType]
																...
																/Window_<YourName>			[WindowType]

	@implements		-
	@base			-

	@devstatus		ready to use
*//*-*************************************************************************************************************/

class SvtViewOptions
{
	//-------------------------------------------------------------------------------------------------------------
	//	public methods
	//-------------------------------------------------------------------------------------------------------------

	public:

		//---------------------------------------------------------------------------------------------------------
		//	constructor / destructor
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		standard constructor and destructor
			@descr		This will de-/initialize an instance with default values.
						You must give us the basic type of your view and a name which specify right entry
						in dynamical configuration list. If entry not exist, we create a new one!

			@seealso	enum EViewType

			@param		"eType" specify type of your view and is used to use right data container!
			@param		"sViewName" specify the name of your view and is the key name in data list too.
			@return		-

			@onerror	An assertion is thrown in debug version. Otherwise we do nothing!
		*//*-*****************************************************************************************************/

         SvtViewOptions( EViewType eType, const ::rtl::OUString& sViewName );
        ~SvtViewOptions();

        static void            AcquireOptions();
        static void            ReleaseOptions();

		//---------------------------------------------------------------------------------------------------------
		//	interface
		//---------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		use it to get information about existing entries in configuration
			@descr		The methods to set/get the position or size will create a new entry automaticly if
						it not already exist and work with default values!
						If this a problem for you - you MUST call these method before and
						you must make up your own mind about that.

			@seealso	method Get/Set-Position/Size()

			@onerror	No error should occure.
		*//*-*****************************************************************************************************/

		sal_Bool Exists() const;

		/*-****************************************************************************************************//**
			@short		use it to delete an entry of dynamic view set
			@descr		You can use this method to delete anexisting node in configuration.
						But if you call a Set- or Get- method again on this instance
						the item is created again! If you do nothing after this call
						your view will die relay in configuration ...

			@seealso	method Get/Set-Position/Size()
			@seealso	method Exist()

			@return		True if delete OK, False otherwise.

			@onerror	We return sal_False.
		*//*-*****************************************************************************************************/

		sal_Bool Delete();

		/*-****************************************************************************************************//**
			@short		use it to set/get position of your view
			@descr		It's only supported for:	- dialogs
													- tab-dialogs
													- windows
						If you call it for "tab-pages" you will get an assertion in debug version.
						In a product version we do nothing!

			@seealso	-

			@onerror	An assertion is thrown in debug version. Otherwise we do nothing!
		*//*-*****************************************************************************************************/

		void GetPosition( sal_Int32& nX, sal_Int32& nY ) const	;
		void SetPosition( sal_Int32  nX, sal_Int32  nY )		;

		/*-****************************************************************************************************//**
			@short		use it to set/get size of your view
			@descr		It's only supported for:	- dialogs
													- windows
						If you call it for "tab-dialogs" or "tab-pages" you will get an assertion in debug version.
						In a product version we do nothing!

			@seealso	-

			@onerror	An assertion is thrown in debug version. Otherwise we do nothing!
		*//*-*****************************************************************************************************/

		void GetSize( sal_Int32& nWidth, sal_Int32& nHeight ) const	;
		void SetSize( sal_Int32  nWidth, sal_Int32  nHeight )		;

		/*-****************************************************************************************************//**
			@short		use it to set/get the page number which was the last active one
			@descr		It's only supported for:	- tab-dialogs
						If you call it for other ones you will get an assertion in debug version.
						In a product version we do nothing!

			@seealso	-

			@onerror	An assertion is thrown in debug version. Otherwise we do nothing!
		*//*-*****************************************************************************************************/

		sal_Int32	GetPageID(					) const	;
		void		SetPageID( sal_Int32 nID	)		;

		/*-****************************************************************************************************//**
			@short		use it to set/get the visual state of a window
			@descr		It's only supported for:	- windows
						If you call it for other ones you will get an assertion in debug version.
						In a product version we do nothing!

			@seealso	-

			@onerror	An assertion is thrown in debug version. Otherwise we do nothing!
		*//*-*****************************************************************************************************/

		sal_Bool	IsVisible	(					) const	;
		void		SetVisible	( sal_Bool bState	)		;

		/*-****************************************************************************************************//**
			@short		use it to set/get the user data
			@descr		It's supported for ALL types!
						Every view can handle his own user defined data set.
						We support one string - his content is your problem!
						But you can use our static method to seperate or join values from/to these string ...

			@seealso	method SeperateUserData()
			@seealso	method JoinUserData()

			@onerror	No error should occure.
		*//*-*****************************************************************************************************/

		::rtl::OUString	GetUserData(								) const	;
		void			SetUserData( const ::rtl::OUString& sData	)		;

		/*-****************************************************************************************************//**
			@short		use it to seperate all token from user data string or build a new one
			@descr		These are helper methods for Get/SetUserData()

			@seealso	method GetUserData()
			@seealso	method SetUserData()

			@onerror	No error should occure.
		*//*-*****************************************************************************************************/

		static ::com::sun::star::uno::Sequence< ::rtl::OUString >	SeperateUserData( const ::rtl::OUString&									sData	, sal_Unicode cSeperator = ';' );
		static ::rtl::OUString										GenerateUserData( const ::com::sun::star::uno::Sequence< ::rtl::OUString >&	seqData	, sal_Unicode cSeperator = ';' );

	//-------------------------------------------------------------------------------------------------------------
	//	private methods
	//-------------------------------------------------------------------------------------------------------------

		/*-****************************************************************************************************//**
			@short		return a reference to a static mutex
			@descr		These class is threadsafe.
						We create a static mutex only for one time and use it to protect our refcount and container
						member!

			@seealso	-

			@param		-
			@return		A reference to a static mutex member.

			@onerror	-
		*//*-*****************************************************************************************************/

		static ::osl::Mutex& GetOwnStaticMutex();

	//-------------------------------------------------------------------------------------------------------------
	//	private member
	//-------------------------------------------------------------------------------------------------------------

	private:

		/// specify which list of views in configuration is used! This can't be a static value!!!
		/// ... because we need this value to work with right static data container.
		EViewType			m_eViewType		;
		::rtl::OUString		m_sViewName		;

		/// - impl. data container as dynamic pointer for smaller memory requirements!
		/// - internal ref count mechanism

		/*Attention

			Don't initialize these static member in these header!
			a) Double defined symbols will be detected ...
			b) and unresolved externals exist at linking time.
			Do it in your source only.
		 */

    	static SvtViewDialogOptions_Impl*		m_pDataContainer_Dialogs	;	/// hold data for all dialogs
		static sal_Int32						m_nRefCount_Dialogs			;
    	static SvtViewTabDialogOptions_Impl*	m_pDataContainer_TabDialogs	;	/// hold data for all tab-dialogs
		static sal_Int32						m_nRefCount_TabDialogs		;
    	static SvtViewTabPageOptions_Impl*		m_pDataContainer_TabPages	;	/// hold data for all tab-pages
		static sal_Int32						m_nRefCount_TabPages		;
    	static SvtViewWindowOptions_Impl*		m_pDataContainer_Windows	;	/// hold data for all windows
		static sal_Int32						m_nRefCount_Windows			;

};		// class SvtViewOptions

#endif	// #ifndef INCLUDED_SVTOOLS_VIEWOPTIONS_HXX
