/**************************************************************************
#*
#*	  last change	$Author: kr $ $Date: 2001/05/04 07:05:18 $
#*	  $Revision: 1.1 $
#*
#*	  $Logfile: $
#*
#*	  Copyright (c) 1989 - 2000, Star Office GmbH
#*
#************************************************************************/

#include <osl/diagnose.h>

#include <cppuhelper/factory.hxx>
#include <cppuhelper/implbase2.hxx>
#include <cppuhelper/servicefactory.hxx>

#include <com/sun/star/registry/XRegistryKey.hpp>
#include <com/sun/star/lang/XServiceInfo.hpp>
#include <com/sun/star/lang/XComponent.hpp>
#include <com/sun/star/lang/XMain.hpp>

using namespace rtl;
using namespace cppu;
using namespace com::sun::star::uno;
using namespace com::sun::star::lang;
using namespace com::sun::star::registry;

#define SERVICENAME		"com.sun.star.test.TestFramework"
#define IMPLNAME		"com.sun.star.comp.TestFramework"


// Implementation in testimpl.cxx
sal_Int32 runTest( const Sequence< OUString > & rArgs, Reference< XMultiServiceFactory > xSMgr ) 
	throw (RuntimeException);

namespace test_framework
{

//--------------------------------------------------------------------------------------------------
inline static Sequence< OUString > getSupportedServiceNames()
{
	OUString aName( RTL_CONSTASCII_USTRINGPARAM(SERVICENAME) );
	return Sequence< OUString >( &aName, 1 );
}

//==================================================================================================
class TestComponentImpl : public WeakImplHelper2< XServiceInfo, XMain >
{
	Reference< XMultiServiceFactory > mxSMgr;
	
public:
	TestComponentImpl( const Reference< XMultiServiceFactory > & xSMgr );
	virtual ~TestComponentImpl();
	
	// XServiceInfo
	virtual OUString SAL_CALL getImplementationName() throw (RuntimeException);
	virtual sal_Bool SAL_CALL supportsService( const OUString & rServiceName ) throw (RuntimeException);
	virtual Sequence< OUString > SAL_CALL getSupportedServiceNames() throw (RuntimeException);
	
	// XMain
    virtual sal_Int32 SAL_CALL run( const Sequence< OUString > & rArgs ) throw (RuntimeException);
};

//##################################################################################################

//__________________________________________________________________________________________________
TestComponentImpl::TestComponentImpl( const Reference< XMultiServiceFactory > & xSMgr )
	: mxSMgr( xSMgr )
{
}
//__________________________________________________________________________________________________
TestComponentImpl::~TestComponentImpl()
{
}

//==================================================================================================
static Reference< XInterface > SAL_CALL TestComponentImpl_create( const Reference< XMultiServiceFactory > & xSMgr )
{
	return Reference< XInterface >( *new TestComponentImpl( xSMgr ) );
}

// XServiceInfo
//__________________________________________________________________________________________________
OUString TestComponentImpl::getImplementationName()
	throw (RuntimeException)
{
	return OUString( RTL_CONSTASCII_USTRINGPARAM(IMPLNAME) );
}
//__________________________________________________________________________________________________
sal_Bool TestComponentImpl::supportsService( const OUString & rServiceName )
	throw (RuntimeException)
{
	const Sequence< OUString > & rSNL = getSupportedServiceNames();
	const OUString * pArray = rSNL.getConstArray();
	for ( sal_Int32 nPos = rSNL.getLength(); nPos--; )
	{
		if (pArray[nPos] == rServiceName)
			return sal_True;
	}
	return sal_False;
}
//__________________________________________________________________________________________________
Sequence< OUString > TestComponentImpl::getSupportedServiceNames()
	throw (RuntimeException)
{
	return test_framework::getSupportedServiceNames();
}


//XMain
//__________________________________________________________________________________________________
sal_Int32 TestComponentImpl::run( const Sequence< OUString > & rArgs )
	throw (RuntimeException)
{
	sal_Int32 nRet = ::runTest( rArgs, mxSMgr );
	return nRet;
}

}


//##################################################################################################
//##################################################################################################
//##################################################################################################


extern "C"
{
//==================================================================================================
void SAL_CALL component_getImplementationEnvironment(
	const sal_Char ** ppEnvTypeName, uno_Environment ** ppEnv )
{
	*ppEnvTypeName = CPPU_CURRENT_LANGUAGE_BINDING_NAME;
}
//==================================================================================================
sal_Bool SAL_CALL component_writeInfo(
	void * pServiceManager, void * pRegistryKey )
{
	if (pRegistryKey)
	{
		try
		{
			Reference< XRegistryKey > xNewKey(
				reinterpret_cast< XRegistryKey * >( pRegistryKey )->createKey(
					OUString( RTL_CONSTASCII_USTRINGPARAM("/" IMPLNAME "/UNO/SERVICES") ) ) );
			xNewKey->createKey( OUString( RTL_CONSTASCII_USTRINGPARAM(SERVICENAME) ) );
			
			return sal_True;
		}
		catch (InvalidRegistryException &)
		{
			OSL_ENSURE( sal_False, "### InvalidRegistryException!" );
		}
	}
	return sal_False;
}
//==================================================================================================
void * SAL_CALL component_getFactory(
	const sal_Char * pImplName, void * pServiceManager, void * pRegistryKey )
{
	void * pRet = 0;
	
	if (pServiceManager && rtl_str_compare( pImplName, IMPLNAME ) == 0)
	{
		Reference< XSingleServiceFactory > xFactory( createSingleFactory(
			reinterpret_cast< XMultiServiceFactory * >( pServiceManager ),
			OUString( RTL_CONSTASCII_USTRINGPARAM(IMPLNAME) ),
			test_framework::TestComponentImpl_create,
			test_framework::getSupportedServiceNames() ) );
		
		if (xFactory.is())
		{
			xFactory->acquire();
			pRet = xFactory.get();
		}
	}
	
	return pRet;
}
}


/**************************************************************************

	$Log: testcp.cxx,v $
	Revision 1.1  2001/05/04 07:05:18  kr
	moved from grande to openoffice
	
	Revision 1.3  2001/03/12 16:22:44  jl
	OSL_ENSHURE replaced by OSL_ENSURE
	
	Revision 1.2  2000/07/28 06:58:53  ab
	SAL_CALL added
	
	Revision 1.1  2000/05/31 09:03:59  ab
	component stuff
	
	
	
**************************************************************************/
