/*************************************************************************
 *
 *  $RCSfile: kdeint.cxx,v $
 *
 *  $Revision: 1.5 $
 *
 *  last change: $Author: pl $ $Date: 2001/07/27 08:16:52 $
 *
 *  The Contents of this file are made available subject to the terms of
 *  either of the following licenses
 *
 *         - GNU Lesser General Public License Version 2.1
 *         - Sun Industry Standards Source License Version 1.1
 *
 *  Sun Microsystems Inc., October, 2000
 *
 *  GNU Lesser General Public License Version 2.1
 *  =============================================
 *  Copyright 2000 by Sun Microsystems, Inc.
 *  901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License version 2.1, as published by the Free Software Foundation.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA  02111-1307  USA
 *
 *
 *  Sun Industry Standards Source License Version 1.1
 *  =================================================
 *  The contents of this file are subject to the Sun Industry Standards
 *  Source License Version 1.1 (the "License"); You may not use this file
 *  except in compliance with the License. You may obtain a copy of the
 *  License at http://www.openoffice.org/license.html.
 *
 *  Software provided under this License is provided on an "AS IS" basis,
 *  WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING,
 *  WITHOUT LIMITATION, WARRANTIES THAT THE SOFTWARE IS FREE OF DEFECTS,
 *  MERCHANTABLE, FIT FOR A PARTICULAR PURPOSE, OR NON-INFRINGING.
 *  See the License for the specific provisions governing your rights and
 *  obligations concerning the Software.
 *
 *  The Initial Developer of the Original Code is: Sun Microsystems, Inc.
 *
 *  Copyright: 2000 by Sun Microsystems, Inc.
 *
 *  All Rights Reserved.
 *
 *  Contributor(s): _______________________________________
 *
 *
 ************************************************************************/
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <salunx.h>
#include <X11/Xatom.h>

#include <saldisp.hxx>
#include <salframe.hxx>
#include <kdeint.hxx>
#include <config.hxx>

#include <osl/file.hxx>

#ifndef _STREAM_HXX
#include <tools/stream.hxx>
#endif

#include <tools/fsys.hxx>

#include <tools/urlobj.hxx>

#include <strhelper.hxx>

#define NO_DEFAULT_ICON

using namespace rtl;
using namespace osl;

int   KDEIntegrator::nInstances = 0;

String KDEIntegrator::CreateKDEPath( char* pType )
{
	String aSearchPath;

	String aType( pType, strlen(pType), RTL_TEXTENCODING_ISO_8859_1 );
	if( maLocalKDEDir.Len() )
	{
		aSearchPath += maLocalKDEDir;
		aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "/share/" ) );
		aSearchPath += aType;
	}
	if( maGlobalKDEDir.Len() )
	{
		aSearchPath += ':';
		aSearchPath += maGlobalKDEDir;
		aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "/share/" ) );
		aSearchPath += aType;
	}
	aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( ":/opt/kde/share/" ) );
	aSearchPath += aType;
	aSearchPath.AppendAscii( RTL_CONSTASCII_STRINGPARAM( ":/usr/local/kde/share/" ) );
	aSearchPath += aType;

	return aSearchPath;
}

KDEIntegrator::KDEIntegrator( SalFrame* pFrame ) :
		DtIntegrator( pFrame )
{
	nInstances++;
	meType = DtKDE;
	rtl_TextEncoding aEncoding = gsl_getSystemTextEncoding();
	const char* pEnv = getenv( "KDEDIR" );
	if( pEnv )
		maGlobalKDEDir = String( pEnv, aEncoding );
	pEnv = getenv( "HOME" );
	if( pEnv )
	{
		maLocalKDEDir = String( pEnv, aEncoding );
		maLocalKDEDir.AppendAscii( RTL_CONSTASCII_STRINGPARAM( "/.kde" ) );
	}

	InitMimeTypes();

	ByteString aBinDir( maGlobalKDEDir, aEncoding );
	aBinDir += "/bin";
	ByteString aPath( "PATH=" );
	pEnv = getenv( "PATH" );
	if( pEnv )
		aPath += pEnv;
	if( aPath.Search( aBinDir ) == STRING_NOTFOUND )
	{
		aPath += ':';
		aPath += aBinDir;
		putenv( const_cast<char*>(aPath.GetBuffer()) );
	}
	mnCurrentDesktopAtom	= 0;
	mnCurrentDesktop		= 0;

	// get locale
	pEnv = getenv( "HOME" );
	ByteString aKDErc;
	if( pEnv )
	{
		aKDErc = pEnv;
		aKDErc += "/.kderc";
	}
	SvFileStream aStream( String( aKDErc, aEncoding ), STREAM_READ );
	if( aStream.IsOpen() )
	{
		while( ! aStream.IsEof() &&
			   aKDErc.CompareIgnoreCaseToAscii( "language=", 9 ) != COMPARE_EQUAL )
			aStream.ReadLine( aKDErc );
		if( aKDErc.CompareIgnoreCaseToAscii( "language=", 9 ) == COMPARE_EQUAL )
		{
			aKDErc.Erase( 0, 9 );
			maKDELanguage = aKDErc.GetToken( 0, ':' );
		}
	}
	if( maKDELanguage == "C" )
		maKDELanguage.Erase();
}

KDEIntegrator::~KDEIntegrator()
{
	nInstances--;
	
	// empty the mime type list
	while( maMimeTypes.Count() )
		delete maMimeTypes.Remove( (ULONG)0 );
	
}

KDEMimeType* KDEIntegrator::FindTypeForFile( const String& rFile )
{
	if( maMimeTypes.Count() )
	{
		String aExtension;
		String aFile( rFile );
		int nToken = aFile.GetTokenCount( '.' );
		if( nToken > 1 )
		{
			aExtension = aFile.GetToken( nToken-1, '.' );
			for( int i = 0; i < maMimeTypes.Count(); i++ )
			{
				KDEMimeType* pType = maMimeTypes.GetObject( i );
				nToken = pType->maExtensions.GetTokenCount( ':' );
				for( int n=0; n < nToken; n++ )
				{
					if( pType->maExtensions.GetToken( n, ':' ) == aExtension )
						return pType;
				}
			}
		}
	}
	return 0;
}

void KDEIntegrator::IntegrateMimePath( const String& rDir )
{
	rtl_TextEncoding aEncoding = gsl_getSystemTextEncoding();
	OUString aDirPath;
	OSL_VERIFY( ::osl::FileBase::getFileURLFromSystemPath( rDir , aDirPath ) == osl_File_E_None);
	Directory aDir( aDirPath );
	aDir.open();
	DirectoryItem aItem;

	while( aDir.getNextItem( aItem ) == FileBase::E_None )
	{
		FileStatus aStatus( FileStatusMask_FileName			|
							FileStatusMask_FileURL          |
							FileStatusMask_LinkTargetURL    |
							FileStatusMask_Type
							);
		if( aItem.getFileStatus( aStatus ) == FileBase::E_None )
		{
			String aFileName = aStatus.getFileName();
			if( aFileName.EqualsAscii( "." ) || aFileName.EqualsAscii( ".." ) )
				continue;
			
			OUString aSysPath;
			OSL_VERIFY( osl_File_E_None == 
				::osl::FileBase::getSystemPathFromFileURL( aStatus.getLinkTargetURL(), aSysPath ));
			if( aStatus.getFileType() == FileStatus::Directory )
			{
				IntegrateMimePath( aSysPath );
				continue;
			}
			
			if( ! aFileName.Copy( aFileName.Len()-7 ).EqualsAscii( ".kdelnk" ) )
				continue;
			
			SvFileStream aStream( aSysPath, STREAM_READ );
			if( ! aStream.IsOpen() )
				continue;
			
			ByteString aLine, aKey, aValue;
			KDEMimeType* pType = new KDEMimeType;
			BOOL bInsert = FALSE;
			while( ! aStream.IsEof() )
			{
				aStream.ReadLine( aLine );
				aKey = WhitespaceToSpace( aLine.GetToken( 0, '=' ) );
				aValue = WhitespaceToSpace( aLine.Copy( aKey.Len()+1 ) );
				if( aKey.CompareIgnoreCaseToAscii( "mimetype" ) == COMPARE_EQUAL )
					pType->maName = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "icon" ) == COMPARE_EQUAL )
					pType->maIcon = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "miniicon" ) == COMPARE_EQUAL )
					pType->maMiniIcon = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "type" ) == COMPARE_EQUAL &&
						 aValue.CompareIgnoreCaseToAscii( "mimetype" ) == COMPARE_EQUAL )
					bInsert = TRUE;
				else if( aKey.CompareIgnoreCaseToAscii( "defaultapp" ) == COMPARE_EQUAL )
					pType->maApplication = String( aValue, aEncoding );
				else if( aKey.CompareIgnoreCaseToAscii( "patterns" ) == COMPARE_EQUAL && aValue.Len() )
				{
					aValue.SearchAndReplaceAll( ';', ':' );
					aValue.SearchAndReplaceAll( "*.", "" );
					
					if( aValue.Len() && aValue.GetChar( aValue.Len()-1 ) == ':' )
						aValue.Erase( aValue.Len()-1 );
					
					pType->maExtensions = String( aValue, aEncoding );
				}
			}
			if( bInsert && pType-> maExtensions.Len() )
				maMimeTypes.Insert( pType, LIST_APPEND );
			else
				delete pType;
		}
	}
}

void KDEIntegrator::InitMimeTypes()
{
	int i, nPaths;

	String aSearchPath = CreateKDEPath( "mimelnk" );
	nPaths = aSearchPath.GetTokenCount( ':' );
	for( i = 0; i < nPaths; i++ )
	{
		String aPath = aSearchPath.GetToken( i, ':' );
		IntegrateMimePath( aPath );
	}
}

BOOL KDEIntegrator::StartProcess( String& rFileName, String& rParam
, const String& rDir )
{
	String aFiles( rFileName );
	if( rParam.Len() )
	{
		aFiles += ' ';
		aFiles += rParam;
	}
	
	KDEMimeType* pType = FindTypeForFile( rFileName );
	if( pType )
	{
		aFiles.Insert( ' ', 0 );
		aFiles.Insert( pType->maApplication, 0 );
	}
	
	return LaunchProcess( aFiles, rDir );
}

BOOL KDEIntegrator::GetSystemLook( SystemLookInfo& rInfo )
{
	BOOL bRet = FALSE;
    rtl_TextEncoding aEncoding = osl_getThreadTextEncoding();

	String aFile( aHomeDir );
	aFile.AppendAscii("/.kderc");

    if( access( ByteString( aFile, aEncoding ).GetBuffer(), F_OK ) )
    {
        aFile = aHomeDir;
        aFile.AppendAscii("/.kde2/share/config/kdeglobals");
        if( access( ByteString( aFile, aEncoding ).GetBuffer(), F_OK ) )
            return FALSE;
    }

	Config aConfig( aFile );

	ByteString aLine;
	if( aConfig.HasGroup( "WM" ) )
	{
		aConfig.SetGroup( "WM" );
		bRet = TRUE;

		aLine = aConfig.ReadKey( "activeBackground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowActiveStart =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
				   	   aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "activeBlend" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowActiveEnd =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
				   	   aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "inactiveBackground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowInactiveStart =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
					   aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "inactiveBlend" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.windowInactiveEnd =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
					   aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "inactiveForeground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.inactiveForeground =
				Color( aLine.GetToken( 0, ',' ).ToInt32(),
					   aLine.GetToken( 1, ',' ).ToInt32(),
					   aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "activeForeground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.activeForeground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "titleFont" );
		rInfo.windowFont = String( aLine, gsl_getSystemTextEncoding() );
	}

	if( aConfig.HasGroup( "General" ) )
	{
		aConfig.SetGroup( "General" );
		bRet = TRUE;

		aLine = aConfig.ReadKey( "foreground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.foreground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "background" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.background = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "selectForeground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.selectForeground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );
		aLine = aConfig.ReadKey( "selectBackground" );
		if( aLine.GetTokenCount( ',' ) >= 3 )
			rInfo.selectBackground = Color(
				aLine.GetToken( 0, ',' ).ToInt32(),
				aLine.GetToken( 1, ',' ).ToInt32(),
				aLine.GetToken( 2, ',' ).ToInt32() );

		aLine = aConfig.ReadKey( "font" );
		rInfo.controlFont = String( aLine, gsl_getSystemTextEncoding() );
	}

	return bRet;
}
